# frozen_string_literal: true

module Hermes
  # Nud class
  class Nud
    # Linked to the method of housing_request :
    #   - nud_creations
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::ActivableConcern
    include Hermes::Concerns::SopraXmlSender
    
    field :applicant_id, type: BSON::ObjectId
    field :department, type: String
    field :creation_date, type: DateTime
    field :unique_number, type: String
    field :provider, type: String
    
    writer_auth_roles 'collaborateur entreprise', "collaborateur offre logement", 'superadministrateur'
    
    belongs_to(
      :housing_request,
      class_name: 'Hermes::HousingRequest',
      index: true, touch: true
    )
    
    belongs_to(
      :applicant,
      class_name: 'Hermes::Applicant',
      index: true, touch: true
    )
    validates_presence_of :housing_request
    validate :custom_unique_number_unicity
    
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id
    
    

    common_indexes *%i[
      department
      creation_date
      unique_number
      is_active
      provider
    ]
    
    before_save :before_save_actions
    after_save      :after_save_actions
    after_create :after_create_actions
    after_destroy :after_destroy_actions
    
    ###############################
    #### Notification #############
    include ::Concerns::NotificationCenter::Notifiable
    
    def self.targets_selector(nud, opt = {})
      [nud.applicant]
    end
    
    def notifiable_title(key, opt = {})
      I18n.t("#{key}.title")
    end
    
    def notifiable_body(key, opt = {})
      I18n.t("#{key}.body", nud: unique_number)
    end
    
    def notifiable_path
      "https://als-hermes-int.be-idys.com/nuds/#{unique_number}"
    end
    
    def notification_mailer
      ::Hermes::NotificationCenter::HermesMailer
    end
    
    def update_provider
      if HERMES_CONFIG["DEP_PROVIDERS"]
        HERMES_CONFIG["DEP_PROVIDERS"].each do |key, val|
          val.each do |dep|
            if (unique_number[0..2] === (dep.length == 2 ? "0" + dep : dep))
              self.provider = key
            end
          end
        end
      end
    end
    
    def update_all_nud_providers
      nuds = Hermes::Nud.where(provider: nil)
      nuds.length
      
      nuds.each_with_index do |nud, i|
        if nud.unique_number
          nud.update_provider
          nud.save
          
          ap i
        else
          ap nud
        end
      end
    end
    
    def update_housing_request_unique_numbers
      housing_request&.set(unique_numbers: housing_request.nuds.pluck(:unique_number).uniq, updated_at: DateTime.now)
    end

    def remove_housing_request_unique_numbers
      housing_request&.set(unique_numbers: housing_request.unique_numbers - [unique_number], updated_at: DateTime.now)
    end 
    
    ###############################
    ###############################
    
    # ========================== #
    # - SIDE EFFETCTS HANDLERS - #
    # ========================== #
    def before_save_actions
      update_provider if unique_number_changed?
      update_applicant_id if housing_request_id_changed?
    end

    def after_create_actions
      housing_request.save
      sne_files_sync
      notify(key: NOTIFICATION_KEYS[:nud][:create], channels: ['email'])
    end
    
    def after_save_actions
      update_housing_request_unique_numbers if unique_number_changed?
      update_housing_request_sne_created_at if creation_date_changed?
    end
    
    def after_update_actions
      housing_request.save unless applicant_id_changed? # to avoid infinit loop
    end
    
    def after_destroy_actions
      remove_housing_request_unique_numbers
    end
    
    # ============================== #
    # - SIDE EFFETCTS DEPENDENCIES - #
    # ============================== #
    def custom_unique_number_unicity
      nuds = Hermes::Nud.where(unique_number: unique_number, :id.ne => id)
      if housing_request.is_active && nuds.any? { |nud| nud.housing_request.is_active == true }
        errors.add :unique_number, 'Ce numéro unique est déja attribué à une housing_request avec is_active=true' 
      end 
      
    end

    def update_applicant_id
      self.applicant_id = housing_request.main_applicant_id
    end

    def update_housing_request_sne_created_at
      if housing_request_id && housing_request && creation_date
        housing_request.update(sne_created_at: creation_date)
      end 
    end 
    
    def oldest_for_department?
      # has the oldest @creation_date
      # WITHIN @demartement.nuds
      # WHERE_SAME @housing_request
      oldest_other = housing_request.nuds
                       .where(
                         id: {:$ne => id},
                         department: department,
                         creation_date: {:$lte => creation_date}
                       ).order_by(creation_date: 1)
                       &.first
                       &.creation_date
      oldest_other.blank? || oldest_other > creation_date
    end
    
    def sne_files_sync
      if housing_request
        docs = housing_request&.housing_request_folder&.required_documents
        docs ||= []
        docs.each do |doc|
          is_pending_doc = (doc.sne_code.present? && doc.sne_category.present? && doc.doc_id.present? && !doc.exists_in_sne)
          doc.upload_file_to_sne if is_pending_doc
        end
      end
    end
  end
end
