require 'rails_helper'
module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    applicant_id: 'String',
    identification_number: 'String',
    civility: 'String',
    first_name: 'String',
    last_name: 'String',
    birth_date: 'Date',
    validation: 'Boolean',
    contract_start_date: 'Date',
    contract_end_date: 'Date'
  }

  ACCEPTABLE_PARAMS = Api::Dmo::EntityValidationsController::ACCEPTABLE_PARAMS

  VALID_OPERATORS = Api::Dmo::EntityValidationsController::VALID_OPERATORS

  describe Api::Dmo::EntityValidationsController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:entity) { create(:entity, user: responsible) }
    let!(:entity_validation) { create(:entity_validation, entity: entity) }
    let!(:applicant) { create(:applicant) }

    describe 'GET /entity_validations' do
      before :each do
        authenticated_header(request, responsible)
        get 'index'
      end

      it 'returns status code 200' do
        expect(response).to have_http_status(200)
        expect(json['data'].map{ |ev| ev['id'] }).to include(entity_validation.id.to_s)
      end
    end

      describe 'POST /entity_validation' do
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                    applicant_id: applicant.id.to_s,
                    identification_number: 'SFD54EHJS87JD',
                    contract_start_date: (Date.today - 3.days)
                  }
              }
          }
        end

      it 'returns status code 201' do
        expect(response).to have_http_status(201)
        expect(json['data']['attributes']['active']).to be(true)
      end
    end

    MUTATE_PARAMS.each do |param, type|
      describe 'put request with ' + param.to_s + ' (' + type.to_s + ') is valid' do
        let(:params) do
          attr = {}
          attr[param] = get_typed_value(type)
          attr
        end
        before :each do
          authenticated_header(request, responsible)
          put 'update', {
              id: entity_validation.id,
              data: {
                  attributes: params
              }
          }
        end
        it 'returns status code 201 and the value appears in json' do
          expect(response).to have_http_status(200)
          if (type == 'Date' || type == 'DateTime')
            expect(json['data']['attributes'][param.to_s]).not_to be_empty
            expect(json['data']['attributes'][param.to_s]).not_to eq('')
          else
            expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type))
          end
        end
      end
    end
  end

  describe Api::Dmo::EntityValidationsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    require_relative '../../swagger_helper'
    TAGS_API_DMO_ENTITY_VALIDATION = 'API_DMO_ENTITY_VALIDATIONS'
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:entity) { create(:entity, user: responsible) }

    context 'EntityValidations' do
      path '/api/dmo/entity_validations' do
        get 'Retrieves all entity_validations' do
          tags TAGS_API_DMO_ENTITY_VALIDATION
          produces 'application/json'
          security [Bearer: {}]
          parameter name: :entity_id, in: :query, required: false, description: "Si fournit, entity_id sera utilisé pour recherché l'entité en question (à condition d'avoir les droits d'accès) sinon ce sera l'entité lié au current_user qui sera utilisé"
          paginator
          
          response '200', 'entity_validation found' do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let!(:entity) { create(:entity, user: responsible) }
            let!(:entity_validation) { create_list(:entity_validation, 5, entity: entity) }
            
            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) { entity_validation }
            end
          end

          response '404', "entity not found (cela se produit lorsque le current_user n'est pas relié a une entité ou que le current_user n'a pas les droits sur l'entité fournit (entity_id) " do
            let!(:admin) {create(:user, roles: ['Administrator'])}
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            run_test!
          end
        end

        post 'Create an new entity_validation' do
          tags TAGS_API_DMO_ENTITY_VALIDATION
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, value|
            json_params[key] = nice_typev2(value)
          end
          required = %i[identification_number applicant_id contract_start_date]

          parameter name: :params, in: :body, schema: jsonify_params(json_params, required: required)
          parameter name: :entity_id, in: :body, required: false, description: "Si fournit, entity_id sera utilisé pour recherché l'entité en question (à condition d'avoir les droits d'accès) sinon ce sera l'entité lié au current_user qui sera utilisé"
          
          response '201', 'entity_validation created' do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let(:applicant) { create(:applicant) }
            let(:identification_number) { 'ANDJD866DHSJ' }
            let(:applicant_id) { applicant.id.to_s }
            let(:contract_start_date) { (Date.today - 3.days).to_s }
            let(:params) do
              jsonapi_attributes({
                identification_number: identification_number,
                applicant_id: applicant_id,
                contract_start_date: contract_start_date
              })
            end
            
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            after do
              json = JSON.parse(response.body)
              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'entity_validation'
            end
          end

          response '404', "entity not found (cela se produit lorsque le current_user n'est pas relié a une entité ou que le current_user n'a pas les droits sur l'entité fournit (entity_id)" do
            let!(:admin) {create(:user, roles: ['Administrator'])}
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:params) do
              jsonapi_attributes({
                  identification_number: 'JHDUD7657DG',
                  applicant_id: 1,
                  contract_start_date: Date.today.to_s
              })
            end
            run_test!
          end

          response '422', "missing or invalid required fields [:identification_number, :applicant_id, :contract_start_date]" do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let(:applicant) { create(:applicant) }
            let(:identification_number) { nil }
            let(:applicant_id) { nil }
            let(:contract_start_date) { nil }
            let(:params) do
              jsonapi_attributes({
                identification_number: identification_number,
                applicant_id: applicant_id,
                contract_start_date: contract_start_date
              })
            end
            run_test!
          end
        end
      end

      path '/api/dmo/entity_validations/{id}' do
        get 'Retrieves an entity_validation' do
          tags TAGS_API_DMO_ENTITY_VALIDATION
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]

          parameter name: :id, in: :path, required: true

          response '200', 'entity_validation found' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:entity) { create(:entity, user: responsible) }
            let(:entity_validation) { create(:entity_validation, entity: entity) }
            let(:id) { entity_validation.id }

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) { entity_validation }
            end
          end
          response '404', 'entity_validation not found' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:entity) { create(:entity, user: responsible) }
            let(:id) { 1 }

            run_test!
          end
        end

        put 'Update an entity_validation' do
          tags TAGS_API_DMO_ENTITY_VALIDATION
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, value|
            json_params[key] = nice_typev2(value)
          end
          required = %i[identification_number applicant_id contract_start_date]

          parameter name: :id, in: :path, required: true
          parameter name: :params, in: :body, schema: jsonify_params(json_params, required: required)

          response '200', 'entity_validation updated' do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let(:contract_end_date) { (Date.today + 3.days).to_s }
            let!(:entity_validation) { create(:entity_validation, entity: entity) }
            let(:id) { entity_validation.id }
            let(:params) do
              jsonapi_attributes({
                contract_end_date: contract_end_date
              })
            end
            
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            after do
              json = JSON.parse(response.body)
              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'entity_validation'
            end
          end

          response '404', "entity_validation not found ou entity not found (cela se produit lorsque le current_user n'est pas relié a une entité ou que le current_user n'a pas les droits sur l'entité fournit (entity_id)" do
            let!(:admin) {create(:user, roles: ['Administrator'])}
            let(:Authorization) { "Bearer #{token_for(admin)}" }
            let(:id) { 1 }
            let(:params) do
              jsonapi_attributes({})
            end
            run_test!
          end

          response '422', "missing or invalid required fields [:identification_number, :applicant_id, :contract_start_date]" do
            let(:Authorization) { "Bearer #{token_for(responsible)}" }
            let(:identification_number) { nil }
            let(:applicant_id) { nil }
            let(:contract_start_date) { nil }
            let(:entity_validation) { create(:entity_validation, entity: entity) }
            let(:id) { entity_validation.id }
            let(:params) do
              jsonapi_attributes({
                identification_number: identification_number,
                applicant_id: applicant_id,
                contract_start_date: contract_start_date
              })
            end
            run_test!
          end
        end

        delete 'Delete an entity_validation' do
          tags TAGS_API_DMO_ENTITY_VALIDATION
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]

          parameter name: :id, in: :path, required: true

          response '204', 'entity_validation deleted' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:entity) { create(:entity, user: responsible) }
            let(:entity_validation) { create(:entity_validation, entity: entity) }
            let(:id) { entity_validation.id }

            run_test!
          end

          response '404', 'entity_validation not found' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:entity) { create(:entity, user: responsible) }
            let(:id) { 1 }

            run_test!
          end
        end
      end
    end
  end
end
