require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    epci: 'Integer',
    max_date: "Date",
    visit_conducted_date: "Date",
    status: "String",
    visit_status: "String",
    last_status_update_date: "Date",
    date_event: "Date", 
    lessor_event_informations: "String",
    rank: "Integer"
  }

  VALID_OPERATORS = %w[
      $in
      $nin
      $ne
      $gt
      $gte
      $lt
      $lte
      $eq
      $regex
      $all
    ].freeze

  ACCEPTABLE_PARAMS = %w[
    housing_request_id
    housing_offer_id
    epci
    max_date
    applicated_at
    effort_rate
    status_updated_at
    status
    visit_status_updated_at
    visit_status
    visit_conducted_date
    date_event
  ].freeze

  describe Api::Dmo::HousingApplicationsController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:superadministrateur) {create(:user, auth_roles: ['superadministrateur'])}
    let!(:user) {create(:user)}
    let!(:user_creator) {create(:user, email: 'johndoe@mail.com')}
    let!(:applicant) {create(:applicant, user: user, first_name: "martin", last_name: "Matin")}

    let!(:housing_offer) {create(:housing_offer, external_ref: "3615")}
    let!(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s, applicant_ids: nil,  is_active: true, user: user_creator)}

    describe 'POST /housing_applications' do
      it 'should has no housing_applciaiton' do
        expect(housing_request.applications_nb).to be 0
      end

      describe 'Create housing_application with minimal valid datas' do
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
            data: {
              attributes: {
                housing_offer_id: housing_offer.id.to_s,
                housing_request_id: housing_request.id.to_s,
              }
            }
          }
        end
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end

        it 'should update housing_request applciations_nb' do
          expect(housing_request.reload.applications_nb).to be 1
        end
      end
    end

    describe 'GET /housing_applications' do
      let!(:applicant1) {create(:applicant)}
      let!(:applicant2) {create(:applicant)}
      let!(:applicant3) {create(:applicant)}
      let!(:housing_request1) {create(:housing_request, main_applicant_id: applicant1.id.to_s, is_active: true)}
      let!(:housing_request2) {create(:housing_request, main_applicant_id: applicant2.id.to_s, is_active: true)}
      let!(:housing_request3) {create(:housing_request, main_applicant_id: applicant3.id.to_s, is_active: true)}

      let!(:housing_application1) do
        create(
          :housing_application,
          housing_offer_id: housing_offer.id.to_s,
          housing_request_id: housing_request.id.to_s
        )
      end

      let!(:housing_application3) do
        create(
          :housing_application,
          housing_offer_id: housing_offer.id.to_s,
          housing_request_id: housing_request2.id.to_s
        )
      end

      let!(:housing_application2) do
        create(
          :housing_application,
          housing_offer_id: housing_offer.id.to_s,
          housing_request_id: housing_request3.id.to_s
        )
      end

      [
        {user: 'responsible', total_objects: 3},
        {user: 'superadministrateur', total_objects: 3},
        {user: 'user', total_objects: 0},
      ].each do |a|
        describe "as #{a[:user]}" do
          before :each do
            authenticated_header(request, eval(a[:user]))
            get 'index'
          end
          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end
          it 'returns a valid json' do
            expect(json).not_to be_empty
            expect(json['data'].length).to eq(a[:total_objects])
            expect(json['meta']['pagination']).to be_present
            expect(json['meta']['pagination']['total_objects']).to eq(a[:total_objects])
          end
        end
      end
    end

    describe 'GET /housing_applications/:id' do
      describe 'verif it work' do
        let!(:housing_application1) {create(:housing_application, housing_offer_id: housing_offer.id.to_s,
                                            housing_request_id: housing_request.id.to_s)}
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: housing_application1.id, join_housing_offer: true}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data']['attributes']['housing_offer']).not_to be_empty
        end
      end
      describe 'verif it work not work with a non authorized user' do
        let!(:housing_application1) {create(:housing_application, housing_offer_id: housing_offer.id.to_s,
                                            housing_request_id: housing_request.id.to_s)}

        before :each do
          authenticated_header(request, user)
          get 'show', {id: housing_application1.id}
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end
    end

    describe 'PATCH /housing_applications/:id' do
      let!(:user2) {create(:user)}
      let!(:housing_application) {create(:housing_application, housing_offer_id: housing_offer.id.to_s,
                                         housing_request_id: housing_request.id.to_s, status: "EN ATTENTE",)}
      before :each do
        authenticated_header(request, responsible)

        patch 'update', {
          id: housing_application.id.to_s,
          data: {
            attributes: {
              status: "DESISTEMENT",
            }
          }
        }
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
    end

    describe 'POST /housing_applications/update_status' do
      let!(:user2) {create(:user)}
      let!(:nud1) do
        create(
          :nud,
          housing_request: housing_request,
          department: '12',
          creation_date: '2016-11-11T14:53:22',
          unique_number: "A1253EF84F3"
        )
      end
      let!(:housing_application) {create(:housing_application, user_id: user2.id.to_s, housing_offer_id: housing_offer.id.to_s,
                                         housing_request_id: housing_request.id.to_s, status: "INSCRIPTION CAL",)}
                                         
      
      before :each do
        authenticated_header(request, responsible)
      end

      it 'with right status' do
        post 'update_status', {
          data: {
            attributes: {
              status: "RETENUE EN CAL",
              unique_number: "A1253EF84F3",
              external_ref: "3615",
              last_status_update_date: '2018-09-28',
              lessor_event_informations: "en mode confinement chemise-slip",
              date_event: '2018-09-26'
            }
          }
        }
        expect(response).to have_http_status(200)
        expect(json["data"]["attributes"]["status"]).to eq("RETENUE EN CAL")
        expect(json["data"]["attributes"]["last_status_update_date"]).to eq('2018-09-28')
        expect(json["data"]["attributes"]["date_event"]).to eq('2018-09-26')
        expect(json["data"]["attributes"]["lessor_event_informations"]).to eq("en mode confinement chemise-slip")
      end

      it 'with right status' do
        post 'update_status', {
          data: {
            attributes: {
              status: "ANNULEE",
              unique_number: "A1253EF84F3",
              external_ref: "3615",
              last_status_update_date: '2018-09-28',
              lessor_event_informations: "en mode confinement chemise-slip"
            }
          }
        }
        expect(response).to have_http_status(422)
      end


      describe 'with archived offers' do 
        let(:external_ref) { 'reference'}
        let!(:archived_offer1) {create(:housing_offer, created_at: DateTime.now + 1.days, offer_status: 'ARCHIVE', external_ref: external_ref)}
        let!(:archived_offer2) {create(:housing_offer, offer_status: 'ARCHIVE', external_ref: external_ref)}
        let!(:normal_offer) {create(:housing_offer, offer_status: 'EN COURS', external_ref: external_ref)}
  
        let!(:housing_application1) {create(:housing_application, user_id: user2.id.to_s, housing_offer: normal_offer.id.to_s,
          housing_request_id: housing_request.id.to_s, status: "INSCRIPTION CAL",)}
        let!(:housing_application2) {create(:housing_application, user_id: user2.id.to_s, housing_offer: archived_offer1.id.to_s,
            housing_request_id: housing_request.id.to_s, status: "INSCRIPTION CAL",)}

        let!(:params) do 
          {
            data: {
              attributes: {
                status: "RETENUE EN CAL",
                unique_number: "A1253EF84F3",
                external_ref: external_ref,
                last_status_update_date: '2018-09-28',
                lessor_event_informations: "en mode confinement chemise-slip",
                date_event: '2018-09-26'
              }
            }
          }
        end 
          
          before :each do
            authenticated_header(request, responsible)
          end
          
        it 'find unarchived offer' do 
          post 'update_status', params
          expect(response).to have_http_status(200)
          expect(json["data"]["id"]).to eq(housing_application1.id.to_s)
          expect(json["data"]["attributes"]["status"]).to eq("RETENUE EN CAL")
        end 

        it 'find newest offer' do 
          normal_offer.update(offer_status: 'ARCHIVE')
          post 'update_status', params
          expect(response).to have_http_status(200)
          expect(json["data"]["id"]).to eq(housing_application2.id.to_s)
          expect(json["data"]["attributes"]["status"]).to eq("RETENUE EN CAL")
        end 
      end 

    end

    describe 'DELETE /housing_applications/:id' do
      let!(:housing_application) {create(:housing_application, housing_offer_id: housing_offer.id.to_s,
                                         housing_request_id: housing_request.id.to_s)}
      it 'returns status code 404' do
        authenticated_header(request, user)
        delete 'destroy', {id: housing_application.id.to_s }
        expect(response).to have_http_status(404)
      end

      it 'returns status code 204' do
        authenticated_header(request, superadministrateur)
        delete 'destroy', {id: housing_application.id.to_s }
        expect(response).to have_http_status(204)
      end
      
    end

    describe 'POST /housing_applications/attribution' do
      let!(:lessor) { create(:lessor, user_id: responsible.id, title: 'Lessor') }
      let!(:applicant) { create(:applicant) }
      let!(:housing_offer) { create(:housing_offer, lessor: lessor, requalification: true) }
      let!(:lessor_objective) {create(:lessor_objective, department: 'acceuillant', user_id: responsible.id.to_s, lessor: lessor, housing_offer_ids: [housing_offer.id.to_s])}

      before :each do
        authenticated_header(request, responsible)
        applicant.housing_requests.delete_all
      end

      it 'should create a new application with existing applicant' do
        post :attribution, data: { attributes: { applicant_id: applicant.id, housing_offer_id: housing_offer.id } }
        expect(response.status).to eq(201)
        expect(json['data']['attributes']['applicant_id']).to eq(applicant.id.to_s)
        expect(json['data']['attributes']['housing_offer_id']).to eq(housing_offer.id.to_s)
        expect(json['data']['attributes']['housing_request_id']).not_to be_nil
        expect(applicant.housing_requests.count).to eq(1)
      end

      it 'should create a new application with non existing applicant' do
        expect {
          post :attribution, data: { attributes: { nud: '1110121021518G1104', birth_date: '1982-10-10', housing_offer_id: housing_offer.id } }
        }.to change{ Hermes::HousingApplication.count }.by 1
        expect(response.status).to eq(201)
        expect(json['data']['attributes']['applicant_id']).not_to be_nil
        expect(json['data']['attributes']['housing_offer_id']).to eq(housing_offer.id.to_s)
        expect(json['data']['attributes']['housing_request_id']).not_to be_nil
      end
    end
  end

  require_relative '../../swagger_helper'

  describe Hermes::Api::Dmo::HousingApplicationsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_HOUSING_APPLICATIONS = 'API_DMO_HOUSING_APPLICATIONS'
    DESCRIPTION_STATUS_HTML= "<h4>Status des candidatures</h4><p>Cette liste est peut &ecirc;tre amen&eacute; &agrave; &eacute;voluer car certaines corrrespondances avec les pr&eacute;c&eacute;dents status n'est pas encore bien &eacute;tablie.</p><p>A pr&eacute;sent il est indispensable de s&eacute;parer les status de commissions des status de visite, il existe donc deux champs sur le mod&egrave;le HousingApplication qui sont <strong>status</strong> et <strong>visit_status</strong></p><p>Actuellement la correspondance avec le code &eacute;venement fonctionne avec les anciens et les nouveaux status.&nbsp;</p><p>Si un anciens status est utilis&eacute; alors il sera convertit suivant ce tableau.</p><p>Le code evenement sera renvoy&eacute; avec les anciens et nouveaux status (si correspondance)</p><table style='height: 433px;' border='1' width='612'><tbody><tr><td style='width: 138px;'>status</td><td style='width: 140px;'>visit_status</td><td style='width: 165px;'>Ancienne correspondance</td><td style='width: 141px;'>code evenement renvoy&eacute;</td></tr><tr><td style='width: 138px;'><div><div>EN ATTENTE</div></div></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'>Waiting</td><td style='width: 141px;'>&nbsp;</td></tr><tr><td style='width: 138px;'>PLO</td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'>PLO</td><td style='width: 141px;'>&nbsp;</td></tr><tr><td style='width: 138px;'><div><div>TRANSMIS BAILLEUR</div></div></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'><div><div>Inscription CAL</div></div></td><td style='width: 141px;'><div><div>INSCAL</div></div></td></tr><tr><td style='width: 138px;'><div><div>RETENUE EN CAL</div></div></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'><div><div>Attribution lot avec rang</div></div></td><td style='width: 141px;'><div><div>DESCAL</div></div></td></tr><tr><td style='width: 138px;'><div><div>REFUSEE EN CAL</div></div></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'><div><div>Refus attribution</div></div></td><td style='width: 141px;'><div><div>DESCAL03</div></div></td></tr><tr><td style='width: 138px;'><div><div>NON RETENUE</div></div></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'><div><div>Refus lot demandeur</div></div></td><td style='width: 141px;'><div><div>REFLOG</div></div></td></tr><tr><td style='width: 138px;'><div><div>NON RETENUE EN CAL</div></div></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'><div><div>Radiation attribution</div></div></td><td style='width: 141px;'><div><div>RADATT</div></div></td></tr><tr><td style='width: 138px;'><div><div>DESISTEMENT SALARIE AVANT CAL</div></div></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'>&nbsp;</td><td style='width: 141px;'>&nbsp;</td></tr><tr><td style='width: 138px;'><div><div>ANNULEE BAILLEUR</div></div></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'>&nbsp;</td><td style='width: 141px;'>&nbsp;</td></tr><tr><td style='width: 138px;'>&nbsp;<div><div>DESISTEMENT</div></div></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'>&nbsp;</td><td style='width: 141px;'>&nbsp;</td></tr><tr><td style='width: 138px;'>&nbsp;<div><div>ANNULEE</div></div></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'>&nbsp;</td><td style='width: 141px;'>&nbsp;</td></tr><tr><td style='width: 138px;'>&nbsp;<div><div>A L&rsquo;ETUDE ALS</div></div></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'>&nbsp;</td><td style='width: 141px;'>&nbsp;</td></tr><tr><td style='width: 138px;'>&nbsp;<div><div>DESISTEMENT APRES CAL</div></div></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'>&nbsp;</td><td style='width: 141px;'>&nbsp;</td></tr><tr><td style='width: 138px;'>&nbsp;<div><span style='color: #ff6600;'>RETENUE EN CAL</span></div></td><td style='width: 140px;'>&nbsp;<div><div>VISITE PROPOSEE</div></div></td><td style='width: 165px;'>&nbsp;<div><div>Visite propos&eacute;e</div></div></td><td style='width: 141px;'>&nbsp;<div><div>VISPRO</div></div></td></tr><tr><td style='width: 138px;'><span style='color: #ff6600;'>&nbsp;RETENUE EN CAL</span></td><td style='width: 140px;'>&nbsp;<div><div>VISITE EFFECTUEE</div></div></td><td style='width: 165px;'>&nbsp;<div><div>Visite effectu&eacute;e</div></div></td><td style='width: 141px;'>&nbsp;<div><div>VISEFF</div></div></td></tr><tr><td style='width: 138px;'>&nbsp;</td><td style='width: 140px;'>&nbsp;<div><div>DESISTEMENT APRES VISITE</div></div></td><td style='width: 165px;'>&nbsp;</td><td style='width: 141px;'>&nbsp;</td></tr><tr><td style='width: 138px;'>&nbsp;</td><td style='width: 140px;'><div><div>ACCORD APRES VISITE</div></div></td><td style='width: 165px;'>&nbsp;</td><td style='width: 141px;'>&nbsp;</td></tr><tr><td style='width: 138px;'><span style='color: #ff6600;'>&nbsp;RETENUE EN CAL</span></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'><div><div>Attribution lot</div></div></td><td style='width: 141px;'>&nbsp;<div><div>DESCAL01</div></div></td></tr><tr><td style='width: 138px;'><span style='color: #ff6600;'>&nbsp;RETENUE EN CAL</span></td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'>&nbsp;<div><div>Attribution sous condition</div></div></td><td style='width: 141px;'>&nbsp;<div><div>DESCAL02</div></div></td></tr><tr><td style='width: 138px;'>&nbsp;</td><td style='width: 140px;'>&nbsp;</td><td style='width: 165px;'>&nbsp;</td><td style='width: 141px;'>&nbsp;</td></tr></tbody></table>"

    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
    let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}
    let!(:superadministrateur) {create(:user, auth_roles: ['superadministrateur'])}
    let!(:applicant) {create(:applicant, user: responsible)}
    let!(:housing_offer) {create(:housing_offer, external_ref: "CVL-000014")}
    let!(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s, is_active: true, user: user)}

    context 'as admin' do
      let!(:admin) {create(:administrator)}

      path '/api/dmo/housing_applications' do
        post 'Create housing_application' do
          tags TAGS_API_DMO_HOUSING_APPLICATIONS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          description DESCRIPTION_STATUS_HTML

          json_params = {}

          json_params[:housing_offer_id] = {type: :string}
          json_params[:housing_request_id] = {type: :string}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '201', 'housing_request created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:params) do
              jsonapi_attributes(
                {housing_offer_id: housing_offer.id.to_s,
                 housing_request_id: housing_request.id.to_s}
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_applications'
            end
          end
        end
      end

      path '/api/dmo/housing_applications/update_status' do
        post 'Create housing_application' do
          tags TAGS_API_DMO_HOUSING_APPLICATIONS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          json_params[:unique_number] = {type: :string}
          json_params[:external_ref] = {type: :string}
          json_params[:status] = {type: :string}
          json_params[:last_status_update_date] = {type: :date}
          json_params[:lessor_event_informations] = {type: :string}

          parameter name: :params, in: :body, schema: jsonify_params(json_params)

          response '200', 'housing_application created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let!(:nud1) do
              create(
                :nud,
                housing_request: housing_request,
                department: '12',
                creation_date: '2016-11-11T14:53:22',
                unique_number: "A1253EF84F3"
              )
            end
            let!(:housing_application) {create(:housing_application, user_id: user.id.to_s, housing_offer_id: housing_offer.id.to_s,
                                               housing_request_id: housing_request.id.to_s, status: "INSCRIPTION CAL",)}

            let(:params) do
              jsonapi_attributes(
                {         "unique_number":"A1253EF84F3",
                          "external_ref":"CVL-000014",
                          "status":'RETENUE EN CAL',
                          "visit_status": 'VISITE PROPOSEE',
                          "last_status_update_date":"01/01/0001",
                          "lessor_event_informations":"0"}
              )
            end
           # include_context 'with integration test'
          #  include_context 'valid jsonapi'
           # include_context 'not containings errors'
      run_test!
            after do |_example|
              json = JSON.parse(response.body)
              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_applications'
            end
          end
        end
      end

      path '/api/dmo/housing_applications' do
        get 'Retrieves all housing_applications' do
          tags TAGS_API_DMO_HOUSING_APPLICATIONS
          produces 'application/json'

          parameter name: 'ids[]', in: :query, type: :array, items: {type: :string}, collectionFormat: :multi, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = :string
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end
          # VALID_OPERATORS.each do |valid_operator|
          #  parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
          # end

          security [Bearer: {}]
          paginator

          response '200', 'housing_applications found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let!(:housing_applications) {create_list(:housing_application, 1, housing_offer_id: housing_offer.id.to_s,
                                                     housing_request_id: housing_request.id.to_s
            )}

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) {housing_applications}
            end
          end
        end
      end

      path '/api/dmo/housing_applications/{id}' do
        get 'Retrieves a single housing_application by id' do
          tags TAGS_API_DMO_HOUSING_APPLICATIONS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'housing_application found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:housing_application) {create(:housing_application, housing_offer_id: housing_offer.id.to_s,
                                              housing_request_id: housing_request.id.to_s
            )}
            let(:id) {housing_application.id}

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) {housing_application}
            end
          end
        end
      end

      path '/api/dmo/housing_applications/{id}' do
        put 'edit housing_application' do
          tags TAGS_API_DMO_HOUSING_APPLICATIONS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          json_params[:housing_offer_id] = {type: :string}
          json_params[:housing_request_id] = {type: :string}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          description DESCRIPTION_STATUS_HTML

          parameter name: :params, in: :body, schema: jsonify_params(json_params)
          #jsonapi_params( **json_params )

          response '200', 'housing_application updated' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:housing_application) {create(:housing_application, housing_offer_id: housing_offer.id.to_s,
                                              housing_request_id: housing_request.id.to_s
            )}
            let(:id) {housing_application.id.to_s}
            let(:params) do
              jsonapi_attributes(
                {
                  status: "RETENUE EN CAL"
                }
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_applications'
            end
          end
        end
      end

      path '/api/dmo/housing_applications/{id}' do
        delete 'Delete a single housing_application by id' do
          tags TAGS_API_DMO_HOUSING_APPLICATIONS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'housing_application deleted' do
            let(:Authorization) {"Bearer #{token_for(superadministrateur)}"}
            let(:id) {create(:housing_application, housing_offer_id: housing_offer.id.to_s,
                             housing_request_id: housing_request.id.to_s
            ).id}
            run_test!
          end
        end
      end

      path '/api/dmo/housing_applications/attribution' do
        post 'Attribute selected offer to selected applicant' do
          tags TAGS_API_DMO_HOUSING_APPLICATIONS
          produces 'application/json'
          consumes 'application/json'
          json_params = {}
          json_params[:housing_offer_id] = { type: :string }
          json_params[:applicant_id] = { type: :string }
          json_params[:nud] = { type: :string }
          json_params[:birth_date] = { type: :string }
          parameter name: :params, required: true, in: :body, schema: jsonify_params(json_params)
          parameter name: :housing_offer_id, required: true, in: :body
          parameter name: :applicant_id, in: :body, required: false, description: 'Doit être inclu si nud non fournit'
          parameter name: :nud, in: :body, required: false, description: 'Doit être inclu si applicant_id non fournit'
          parameter name: :birth_date, in: :body, required: false, description: 'Doit être inclu si nud fournit'
          security [Bearer: {}]

          response '201', 'housing_application created' do
            let!(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:housing_offer) { create(:housing_offer, :with_lessor, requalification: true) }
            let!(:housing_offer_id) { housing_offer.id }
            let!(:applicant) { create(:applicant) }
            let!(:applicant_id) { applicant.id }
            let!(:params) do
              jsonapi_attributes({
                housing_offer_id: housing_offer_id,
                applicant_id: applicant_id
              })
            end
            run_test!
          end

          response '201', 'housing_application created' do
            let!(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:housing_offer) { create(:housing_offer, :with_lessor, requalification: true) }
            let!(:housing_offer_id) { housing_offer.id }
            let!(:nud) { '1110121021518G1104' }
            let!(:birth_date) { '1982-10-10' }
            let!(:params) do
              jsonapi_attributes({
                housing_offer_id: housing_offer_id,
                nud: nud,
                birth_date: birth_date
              })
            end
            run_test!
          end

          response '404', 'nud not found' do
            let!(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:housing_offer) { create(:housing_offer, :with_lessor, requalification: true) }
            let!(:housing_offer_id) { housing_offer.id }
            let!(:nud) { '000000000000000000' }
            let!(:birth_date) { '1982-10-10' }
            let!(:params) do
              jsonapi_attributes({
                housing_offer_id: housing_offer_id,
                nud: nud,
                birth_date: birth_date
              })
            end
            run_test!
          end

          response '422', 'invalid birth_date' do
            let!(:Authorization) { "Bearer #{token_for(admin)}" }
            let!(:housing_offer) { create(:housing_offer, :with_lessor, requalification: true) }
            let!(:housing_offer_id) { housing_offer.id }
            let!(:nud) { '1110121021518G1104' }
            let!(:birth_date) { '1992-05-10' }
            let!(:params) do
              jsonapi_attributes({
                housing_offer_id: housing_offer_id,
                nud: nud,
                birth_date: birth_date
              })
            end
            run_test!
          end
        end
      end
    end
  end
end