require 'rails_helper'
require 'json'
require 'vcr_config'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')
  
  MUTATE_PARAMS = {
    backoffice_status: "String",
    is_active: 'Boolean',
    attributed_housing_district: 'String',
    attributed_housing_district_postal_code: 'String',
    asked_mobility: 'Boolean',
    valid_mobility: 'Boolean',
    decider: 'String',
    mobility: 'Boolean',
    seeked_types: 'Array',
    entity_help_required: 'Boolean',
    contract_type: 'String',
    registration_number: 'String',
    priority_points_indexes: 'Array',
    nud: 'String',
    id_counter: 'String',
    id_unique: 'String',
    id_unique_associate: 'String',
    internal_ref: 'String',
    forcing_duplicate: 'Boolean',
    electronical_renewal: 'Boolean',
    ec_law_ressources: 'String',
    is_long_delay_anormal: 'Boolean',
    date_calculator_indicator: 'DateTime',
    with_car_park: 'Boolean',
    widen_request: 'Boolean',
    housing_type_list: 'Array',
    expulsion_judgement_date: 'Date',
    other_reason: 'String',
    work_contract_type: 'Array',
    seeked_localisation_list: 'Array',
    lessor_name: 'String',
    #lessor_siren: 'String',
    social_housing_entry_date: 'Date',
    social_housing_name: 'String',
    test_created_at: 'DateTime',
    sne_number: 'String',
    employee_id: 'String',
    employee_start_date: 'DateTime',
    entity_access_level: 'String',
    employee_unit: 'String',
    tab_state: 'String',
    motivations: 'Array',
    seeking_first_floor: 'Boolean',
    seeking_elevator: 'Boolean',
    seeked_max_rent: 'Integer',
    sne_created_at: 'DateTime',
    sne_creation_date: 'DateTime',
    precisions: 'String',
    has_multiple_attributions_under_conditions: 'Boolean',
    seeked_housing_categories: 'Array',
    seeked_epci_first_quartils: 'Array',
    seeked_quartil_belongings: 'Array',
    dependents_monthly_incomes: 'Hash',
    dependents_monthly_charges: 'Hash',
    submitted_to_cal_nb: 'Integer',
    sent_to_cal_nb: 'Integer',
    rejected_in_cal_nb: 'Integer',
    rejected_after_cal_nb: 'Integer',
    accepted_in_cal_nb: 'Integer',
    radiation_date: 'Date',
    radiation_motive: 'String',
    documents_required: 'Integer',
    documents_uploaded: 'Integer',
    is_house_owner: 'Boolean',
    workplace_insee_code: "String",
    cdii: "Boolean",
  }.freeze
  
  ACCEPTABLE_PARAMS = %w[
    viewed_by_colaborator_offre_logement
    form_status
    request_status
    als_id
    attributed_housing_district
    attributed_housing_district_postal_code
    old_request_status
    request_status_updated_at
    old_entity_status
    updated_entity_status
    entity_status
    dalo
    asked_mobility
    valid_mobility
    decider
    submitted_to_cal_nb
    sent_to_cal_nb
    rejected_in_cal_nb
    rejected_after_cal_nb
    accepted_in_cal_nb
    mobility
    seeked_types
    entity_help_required
    contract_type
    registration_number
    applicant_ids
    entity_validated
    priority_points_indexes
    nud
    nb_of_requests_made
    reference_tax_revenues
    seeked_districts
    single_parent
    id_counter
    id_unique
    id_unique_associate
    internal_ref
    forcing_duplicate
    electronical_renewal
    ec_law_ressources
    is_long_delay_anormal
    date_calculator_indicator
    with_car_park
    widen_request
    housing_type_list
    expulsion_judgement_date
    other_reason
    work_contract_type
    seeked_district_postal_codes
    seeked_departments
    seeked_localisation_list
    dependents
    monthly_revenues
    typology_range
    entity_validated
    coapplicants
    lessor_name
    lessor_siren
    social_housing_entry_date
    social_housing_name
    test_created_at
    sne_number
    employee_id
    employee_start_date
    entity_access_level
    employee_unit
    tab_state
    motivations
    seeking_first_floor
    seeking_elevator
    seeked_max_rent
    sne_created_at
    sne_creation_date
    precisions
    has_multiple_attributions_under_conditions
    seeked_housing_categories
    min_surface
    main_applicant_id
    main_applicant
    requester_firstname
    requester_lastname
    requester_fullname
    requester_email
    entity_id
    coapplicant_ids
    coapplicants
    seeked_epci_first_quartils
    seeked_quartil_belongings
    dependents_monthly_incomes
    dependents_monthly_charges
    housing_offer_ids
    radiation_date
    radiation_motive
    documents_required
    documents_uploaded
    is_house_owner
    last_application_date
    has_priority
    last_entity_code
    cdii
    unique_numbers
    last_update_user_id
    last_update_user_last_name
    last_update_user_first_name
  ].freeze
  
  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $eq
    $regex
    $all
  ].freeze
  
  describe Api::Dmo::HousingRequestsController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper
    
    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:superadministrateur) {create(:user, auth_roles: ['superadministrateur'])}
    let!(:user) {create(:user)}
    let!(:applicant) {create(:applicant)}
    let!(:dc_channel) {create(:housing_channel, key: 'dc')}
    
    describe 'with project oriented roles' do
      let!(:demandeur) {create(:user, auth_roles: ['demandeur'], last_name: 'Doe', first_name: 'John')}
      let!(:demandeur_applicant) {create(:applicant, auto_set_user: false)}
      
      let!(:bailleur) {create(:user, auth_roles: ['bailleur'])}
      let!(:entreprise) {create(:user, auth_roles: ['entreprise'])}
      
      describe 'POST as demandeur' do
        before :each do
          authenticated_header(request, demandeur)
          post 'create', {
            data: {
              attributes: {
                main_applicant_id: demandeur_applicant.id.to_s,
              }
            }
          }
        end
        
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end
        
        it 'returns valid json' do
          attributes = json['data']['attributes']
          expect(attributes['main_applicant_id']).to eq demandeur_applicant.id.to_s
          expect(attributes['last_update_user_id']).to eq demandeur.id.to_s
          expect(demandeur_applicant.reload.main_user_id).to eq demandeur.id
          expect(demandeur.reload.main_applicant_id).to eq demandeur_applicant.id
        end
      end
      
      describe 'POST as entreprise with main_user_id' do
        before :each do
          authenticated_header(request, entreprise)
          post 'create', {
            data: {
              attributes: {
                main_applicant_id: demandeur_applicant.id.to_s,
                main_user_id: demandeur.id.to_s
              }
            }
          }
        end
        
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end
        
        it 'returns valid json' do
          attributes = json['data']['attributes']
          expect(attributes['main_applicant_id']).to eq demandeur_applicant.id.to_s
          expect(demandeur_applicant.reload.main_user_id).to eq demandeur.id
          expect(demandeur.reload.main_applicant_id).to eq demandeur_applicant.id
        end
      end
      
      describe 'POST as entreprise' do
        before :each do
          authenticated_header(request, entreprise)
          post 'create', {
            data: {
              attributes: {
                main_applicant_id: demandeur_applicant.id.to_s
              }
            }
          }
        end
        
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end
        
        it 'returns valid json' do
          attributes = json['data']['attributes']
          expect(attributes['main_applicant_id']).to eq demandeur_applicant.id.to_s
          expect(demandeur_applicant.reload.main_user_id).to eq nil
          expect(demandeur.reload.main_applicant_id).to eq nil
        end
      end
    end
    
    # common CRUD
    describe 'POST /housing_requests' do
      describe 'Create housing_request with minimal valid datas' do
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
            data: {
              attributes: {
                main_applicant_id: applicant.id.to_s
              }
            }
          }
        end
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end
      end
      
      MUTATE_PARAMS.each do |param, type|
        describe 'post request with ' + param.to_s + ' (' + type.to_s + ') is valid' do
          let(:params) do
            attr = {}
            attr[param] = get_typed_value(type)
            attr
          end
          before :each do
            params['main_applicant_id'] = applicant.id.to_s
            authenticated_header(request, responsible)
            post 'create', {
              data: {
                attributes: params
              }
            }
          end
          it 'returns status code 201 and the value appears in json' do
            expect(response).to have_http_status(201)
            if (type == 'Date' || type == 'DateTime')
              expect(json['data']['attributes'][param.to_s]).not_to be_empty
              expect(json['data']['attributes'][param.to_s]).not_to eq('')
            else
              expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type))
            end
          end
        end
      end
      # main_applicant_id:
      describe 'post request with main_applicant_id' do
        let!(:applicant1) {create(:applicant, user_id: user.id)}
        
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
            data: {
              attributes: {
                main_applicant_id: applicant1.id.to_s,
                
              }
            }
          }
        end
        it 'returns status code 201 and the value appears in json' do
          expect(response).to have_http_status(201)
          expect(json['data']['attributes']['main_applicant_id']).to eq(applicant1.id.to_s)
        end
      end
      
      # coapplicants: 'Array'
      describe 'post request with coapplicants' do
        let!(:applicant0) {create(:applicant, user_id: user.id)}
        let!(:applicant1) {create(:applicant, user_id: user.id)}
        let!(:applicant2) {create(:applicant, user_id: user.id)}
        $request_id = nil # to store global value
        
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
            data: {
              attributes: {
                main_applicant_id: applicant0.id.to_s,
                form_status: "coco",
                coapplicants: [{'rel_type' => 'P', 'id' => applicant1.id.to_s}, {'rel_type' => 'P', 'id' => applicant2.id.to_s}],
              }
            }
          }
        end
        
        it 'returns status code 201 and the value appears in json' do
          $request_id = json['data']['id']
          expect(response).to have_http_status(201)
          expect(json['data']['attributes']['applicant_ids']).to include(applicant1.id.to_s).and(include(applicant2.id.to_s))
          expect(json['data']['attributes']['coapplicants']).to eq([{'rel_type' => 'P', 'id' => applicant1.id.to_s}, {'rel_type' => 'P', 'id' => applicant2.id.to_s}])
        end
        
        describe 'destroy previous applicants and set with new' do
          let!(:applicant3) {create(:applicant, user_id: user.id)}
          before :each do
            $request_id = json['data']['id']
            # (1..2).each {|i| send("applicant#{i}").destroy}
            authenticated_header(request, responsible)
            put 'update', {
              id: $request_id,
              data: {
                attributes: {
                  coapplicants: [
                    {'rel_type' => 'P', 'id' => applicant1.id.to_s}
                  ]
                }
              }
            }
          end
          
          it 'returns 200' do
            expect(response).to have_http_status(200)
          end
          
          it 'returns valid json' do
            expect(json['data']).to_not be_empty
            expect(json['data']['id']).to eq $request_id
            attributes = json['data']['attributes']
            expect(attributes['coapplicants'].length).to be 1
            expect(attributes['coapplicants'].first['rel_type']).to eq 'P'
            expect(attributes['coapplicants'].first['id']).to eq applicant1.id.to_s
          end
        end
        describe 'destroy previous applicants and set with new' do
          let!(:applicant3) {create(:applicant, user_id: user.id)}
          
          before :each do
            $request_id = json['data']['id']
            hr = Hermes::HousingRequest.find_by(id: json['data']['id'])
            applicant1.destroy
            applicant2.destroy
            applicant3.destroy
            
            authenticated_header(request, responsible)
            get 'show', {id: $request_id}
          end
          
          it 'returns 200' do
            expect(response).to have_http_status(200)
          end
          
          it 'returns valid json' do
            expect(json['data']).to_not be_empty
            expect(json['data']['id']).to eq $request_id
            attributes = json['data']['attributes']
            expect(attributes['coapplicants'].length).to be 0
          end
        end
      end
    end
    
    describe 'situation faille secu' do
      let!(:user1) {create(:user, auth_roles: ['demandeur'])}
      let!(:user2) {create(:user, auth_roles: ['demandeur'])}
      let!(:entity1) {create(:entity)}
      let!(:applicant1) {create(:applicant, user_id: user1.id, entity_id: entity1.id)}
      let!(:applicant2) {create(:applicant, user_id: user2.id, entity_id: entity1.id)}
      
      let!(:housing_request1) {create(:housing_request,
                                      documents_required: 50,
                                      main_applicant_id: applicant1.id.to_s,
                                      user_id: user1.id,
                                      main_user_id: user1.id,
                                      entity_id: entity1.id,
                                      is_active: true)}
      let!(:housing_request2) {create(:housing_request,
                                      documents_required: 50,
                                      main_applicant_id: applicant2.id.to_s,
                                      user_id: user2.id,
                                      main_user_id: user2.id,
                                      entity_id: entity1.id,
                                      is_active: true)}
      
      describe "as user 1" do
        before :each do
          authenticated_header(request, user1)
          get 'show', {id: "active"}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
          expect(json['data']['id']).to eq housing_request1.id.to_s
        end
      end
      
      describe "as user 2" do
        before :each do
          authenticated_header(request, user2)
          get 'show', {id: "active"}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
          expect(json['data']['id']).to eq housing_request2.id.to_s
        end
      end
    end
    
    describe 'POST /housing_requests/search' do
      let!(:entities) {create_list(:entity, 10)}
      let!(:entity) {create(:entity, siret: '12345678901234')}
      let!(:applicant) {create(:applicant, main_employer_siret: entity.siret)}
      let!(:applicant2) {create(:applicant)}
      let!(:housing_request1) {create(:housing_request, documents_required: 50, main_applicant_id: applicant.id.to_s)}
      let!(:housing_request2) {create(:housing_request, documents_required: 50, main_applicant_id: applicant2.id.to_s)}
      
      before :each do
        entities_ids = entities.map {|e| e.id.to_s} + [entity.id.to_s]
        authenticated_header(request, superadministrateur)
        post 'search', {entity_id: {'$in': entities_ids}}
      end
      
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      
      it 'returns a valid json' do
        expect(json['data'].length).to eq(1)
      end
    end
    
    describe 'GET /housing_requests' do
      let!(:housing_request1) {create(:housing_request, documents_required: 50, main_applicant_id: applicant.id.to_s)}
      let!(:housing_request3) {create(:housing_request, documents_required: 150, main_applicant_id: applicant.id.to_s)}
      let!(:housing_request2) {create(:housing_request, documents_required: 150, main_applicant_id: applicant.id.to_s)}
      let!(:nud1) {create(:nud, user_id: responsible.id.to_s, housing_request: housing_request1, unique_number: "0110418041931AL004")}
      let!(:nud2) {create(:nud, user_id: responsible.id.to_s, housing_request: housing_request2, unique_number: "0110418041931AL003")}
      let!(:nud3) {create(:nud, user_id: responsible.id.to_s, housing_request: housing_request3, unique_number: "0110418041931AL005")}
      
      [
        {user: 'responsible', total_objects: 3},
        {user: 'superadministrateur', total_objects: 3},
        {user: 'user', total_objects: 0},
      ].each do |a|
        describe "as #{a[:user]}" do
          before :each do
            authenticated_header(request, eval(a[:user]))
            get 'index'
          end
          it 'returns status code 200' do
            expect(response).to have_http_status(200)
          end
          it 'returns a valid json' do
            expect(json).not_to be_empty
            if (json["data"][0])
              expect(json['data'][0]["attributes"]["providers"]).to eq (["SNE"])
            end
            expect(json['data'].length).to eq(a[:total_objects])
            expect(json['meta']['pagination']).to be_present
            expect(json['meta']['pagination']['total_objects']).to eq(a[:total_objects])
          end
        end
      end
      describe "as admin" do
        before :each do
          
          authenticated_header(request, superadministrateur)
          get 'index', {unique_numbers: "0110418041931AL005"}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(1)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(1)
        end
      
      end
      describe 'GET with regex on first_name and last_name' do
        let!(:applicant2) {create(:applicant, first_name: 'Jean-Pierre', last_name: 'Dupié', auto_set_user: false)}
        let!(:applicant3) {create(:applicant, first_name: 'Jean', last_name: 'Dupont', auto_set_user: false)}
        let!(:housing_request4) {create(:housing_request, documents_required: 150, main_applicant_id: applicant2.id.to_s)}
        let!(:housing_request5) {create(:housing_request, documents_required: 150, main_applicant_id: applicant3.id.to_s)}
        
        before :each do
          authenticated_header(request, superadministrateur)
          get :index, {
            requester_firstname: {
              '$regex': 'jean-pierré'
            }
          }
        end
        
        it 'should return only one request' do
          expect(json['data'].count).to eq(1)
          expect(json['data'].first['id']).to eq(housing_request4.id.to_s)
        end
      end
      describe "send_flux_30_sync" do
        let!(:applicant2) {create(:applicant, first_name: 'Jean-Pierre', last_name: 'Dupié', auto_set_user: false)}
        
        let!(:housing_request4) {create(:housing_request, documents_required: 150, main_applicant_id: applicant2.id.to_s)}
        before :each do
          VCR.use_cassette("test_flux_30", :match_requests_on => [:method, :uri, :path]) do
            authenticated_header(request, superadministrateur)
            post :send_flux_thirty_sync, {
              id: housing_request4.id.to_s
            }
          end
        end
        it 'should return only ok' do
          
          expect(json["error"]).not_to be nil
        
        end
      end
      describe "send_flux_30" do
        let!(:applicant2) {create(:applicant, first_name: 'Jean-Pierre', last_name: 'Dupié', auto_set_user: false)}
        let!(:applicant3) {create(:applicant, first_name: 'Jean', last_name: 'Dupont', auto_set_user: false)}
        let!(:housing_request4) {create(:housing_request, documents_required: 150, main_applicant_id: applicant2.id.to_s)}
        let!(:housing_request5) {create(:housing_request, documents_required: 150, main_applicant_id: applicant3.id.to_s)}
        before :each do
          authenticated_header(request, superadministrateur)
          post :send_flux_thirty, {
            ids: [housing_request4.id.to_s, housing_request5.id.to_s]
          }
        end
        it 'should return only ok' do
          
          expect(json).to eq ({
            "res" => "2 housing_requests flux 30 sended to the workers"
          })
        
        end
      end
      describe "send_flux_30" do
        let!(:applicant2) {create(:applicant, first_name: 'Jean-Pierre', last_name: 'Dupié', auto_set_user: false)}
        let!(:applicant3) {create(:applicant, first_name: 'Jean', last_name: 'Dupont', auto_set_user: false)}
        let!(:housing_request4) {create(:housing_request, documents_required: 150, main_applicant_id: applicant2.id.to_s)}
        let!(:housing_request5) {create(:housing_request, documents_required: 150, main_applicant_id: applicant3.id.to_s)}
        before :each do
          authenticated_header(request, superadministrateur)
          drain_all_jobs
          post :send_flux_thirty, {
            als_ids: [housing_request4.reload.als_id.to_s, housing_request5.reload.als_id.to_s]
          }
        end
        it 'should return ok' do
          
          expect(json).to eq ({
            "res" => "2 housing_requests flux 30 sended to the workers"
          })
        
        end
      end
      
      describe 'GET available HousingRequest for current lessor' do
        let!(:lessor_user) {create(:user, first_name: 'Jaques', last_name: 'Dupont', email: 'jaques.dupont@gmail.com', roles: ['bailleur'])}
        let!(:lessor) {create(:lessor, user_id: lessor_user.id.to_s)}
        let!(:housing_offer) {create(:housing_offer, lessor: lessor)}
        let!(:housing_request) {create(:housing_request, main_applicant_id: applicant.id, is_active: true)}
        let!(:housing_application) {create(:housing_application, housing_request: housing_request, housing_offer: housing_offer)}
        before :each do
          authenticated_header(request, lessor_user)
          get :index, available: true
        end
        
        it 'should return only requests linked to current lessor offers' do
          expect(json['data'].count).to eq(1)
          expect(json['data'].first['id']).to eq(housing_request.id.to_s)
        end
      end
      
      ACCEPTABLE_PARAMS.each do |acceptable_param|
        type = MUTATE_PARAMS[acceptable_param.to_sym] || MUTATE_PARAMS[acceptable_param.to_s]
        if type
          describe 'GET /housing_requests with sort on field ' + acceptable_param + ' works ' do
            let!(:housing_request3) {create(:housing_request, acceptable_param => get_typed_value(type, 2), main_applicant_id: applicant.id.to_s)}
            let!(:housing_request1) {create(:housing_request, acceptable_param => get_typed_value(type, 0), main_applicant_id: applicant.id.to_s)}
            let!(:housing_request2) {create(:housing_request, acceptable_param => get_typed_value(type, 1), main_applicant_id: applicant.id.to_s)}
            before :each do
              authenticated_header(request, responsible)
              get 'index', {sort: {acceptable_param => 1}}
            end
            it 'returns sorted results' do
              expect(response.status).to eq(200)
              if type == 'DateTime'
                housing_request_attribute = json['data'].map {|unit| DateTime.parse(unit['attributes'][acceptable_param])}
                expect(housing_request_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              elsif type == 'Date'
                housing_request_attribute = json['data'].map {|unit| Date.parse(unit['attributes'][acceptable_param])}
                expect(housing_request_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              else
                housing_request_attribute = json['data'].map {|unit| unit['attributes'][acceptable_param]}
                expect(housing_request_attribute).to eq([get_typed_value(type, 0), get_typed_value(type, 1), get_typed_value(type, 2)])
              end
            end
          end
        end
      end
      
  
     
      
      describe 'GET /housing_requests with requester_email case insensitive' do
        let(:applicant1) {create :applicant, email: 'KevIN34515@Wanadoo.fr'}
        let(:applicant2) {create :applicant, email: 'brandon34@wanadoo.fr'}
        let!(:housing_request1) {create(:housing_request, main_applicant_id: applicant1.id)}
        let!(:housing_request3) {create(:housing_request, main_applicant_id: applicant2.id)}
        
        describe 'lower searching upper' do
        before do
          authenticated_header(request, responsible)
          get 'index', {requester_email: "kevin34515@wanadoo.fr"}
        end
        
        it 'returns results' do
          expect(response.status).to eq(200)
          expect(json['data'].length).to eq(1)
          expect(json['data'][0]['id']).to eq housing_request1.id.to_s
          
        end
      end
      describe 'upper searching lower' do
        before do
          authenticated_header(request, responsible)
          get 'index', {requester_email: "BRANDON34@WANADOO.FR"}
        end
        
        it 'returns results' do
          expect(response.status).to eq(200)
          expect(json['data'].length).to eq(1)
          expect(json['data'][0]['id']).to eq housing_request3.id.to_s
          
        end
      end
      end

      describe 'GET /housing_requests with main_employer_siret' do
        let(:applicant_siret1) {create :applicant, main_employer_siret: '12345600000000'}
        let(:applicant_siret2) {create :applicant, main_employer_siret: '98765400000000'}
        let!(:housing_request1) {create(:housing_request, main_applicant_id: applicant.id)}
        let!(:housing_request3) {create(:housing_request, main_applicant_id: applicant_siret1.id)}
        let!(:housing_request2) {create(:housing_request, main_applicant_id: applicant_siret2.id)}
        
        before do
          authenticated_header(request, responsible)
          get 'index', {main_employer_siret: ['12345600000000', '98765400000000']}
        end
        
        it 'returns results' do
          expect(response.status).to eq(200)
          expect(json['data'].length).to eq(2)
          expect(json['data'][0]['id']).to eq housing_request3.id.to_s
          expect(json['data'][1]['id']).to eq housing_request2.id.to_s
        end
      end
      
      describe 'GET /housing_requests with consent_level' do
        let!(:form_level_1) {create :entity_form_level, level: 2}
        let!(:form_level_2) {create :entity_form_level, level: 4}
        let!(:entity_form_1) {create :entity_form, housing_request_id: housing_request1.id, entity_form_level_id: form_level_1.id}
        let!(:entity_form_2) {create :entity_form, housing_request_id: housing_request2.id, entity_form_level_id: form_level_2.id}
        let!(:entity_form_3) {create :entity_form, housing_request_id: housing_request3.id, level: 2}
        let!(:housing_request4) {create :housing_request, main_applicant: applicant, cdii: true}
        let!(:entity_form_4) {create :entity_form, housing_request_id: housing_request4.id, level: 2}
        
        before do
          authenticated_header(request, responsible)
          get 'index', {consent_level: 2, cdii: {"$ne": true}}
        end
        
        it 'succeeds' do
          expect(response).to have_http_status 200
        end
        
        it 'returns results' do
          expect(json['data'].length).to eq 2
          ids = [housing_request1, housing_request3].map { |hr| hr.id.to_s }
          expect(json['data'].map {|hr| hr['id']}).to eq ids
        end
        
        context 'when entity_id' do
          let!(:entity) {create :entity, siret: '12345678901234'}
          let!(:app4) {create(:applicant, main_employer_siret: entity.siret)}
          let!(:housing_request4) {create :active_request, main_applicant: app4}
          let!(:entity_form_level) {create :entity_form_level, entity_id: entity.id, level: 4}
          let!(:entity_form_4) do
            create :entity_form, housing_request_id: housing_request4.id, entity_form_level_id: entity_form_level.id
          end
          
          before do
            get 'index', {consent_level: { '$in': [3, 4]}, entity_id: {"$in": [entity.id.to_s]}}
          end
          
          it 'succeeds' do
            expect(response).to have_http_status 200
          end
          
          it 'returns results' do
            expect(json['data'].length).to eq 1
            expect(json['data'].map {|hr| hr['id']}).to eq [housing_request4.id.to_s]
          end
        end
      end
    end
    
    describe 'GET /housing_requests/:id' do
      describe 'verif it work' do
        let!(:housing_request1) {create(:housing_request, motivations: ['acceuillant'], documents_required: 50, main_applicant_id: applicant.id.to_s)}
        
        it_behaves_like "true user all allowed" do
          let(:path) {['show', {id: housing_request1.id}]}
        end
        
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: housing_request1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end
      describe 'get active housing request for current_user' do
        
        let!(:demandeur) {create(:user, auth_roles: ['demandeur'], last_name: 'Doe', first_name: 'John')}
        let!(:demandeur_applicant) {create(:applicant, user: demandeur)}
        let!(:housing_request1) {create(:housing_request, is_active: true, main_user_id: demandeur.id, responsible: demandeur.id, motivations: ['acceuillant'], documents_required: 50, main_applicant_id: demandeur_applicant.id.to_s)}
        
        before :each do
          authenticated_header(request, demandeur)
          get 'show', {id: 'active'}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json["data"]["id"]).to eq(housing_request1.id.to_s)
          expect(json["data"]["attributes"]["motivations"]).to eq(['acceuillant'])
        end
      end
      describe 'check is nud is rendrered' do
        let!(:housing_request1) {create(:housing_request, documents_required: 50, main_applicant_id: applicant.id.to_s)}
        let!(:nud1) {create(:nud, user_id: responsible.id.to_s, housing_request: housing_request1, unique_number: "ZERFEF")}
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: housing_request1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json["data"]["attributes"]["unique_numbers"]).to eq(["ZERFEF"])
        end
      end
      describe 'verif it work not work with a non authorized user' do
        let!(:housing_request1) {create(:housing_request,  documents_required: 50, main_applicant_id: applicant.id.to_s)}
        
        before :each do
          authenticated_header(request, user)
          get 'show', {id: housing_request1.id}
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end
    end
    
    describe 'patch request with entity_validated is valid' do
      let!(:housing_request) {create(:housing_request, user: user2, main_applicant_id: applicant.id.to_s)}
      let!(:user2) {create(:user, auth_roles: ["Entreprise"])}
      let!(:entity) {create(:entity, user_id: user2.id, siret: '12345678901234')}
      
      before :each do
        applicant.update(main_employer_siret: entity.siret)
        authenticated_header(request, user2)
        patch 'update', {
          id: housing_request.id.to_s,
          data: {
            attributes: {entity_validated: true}
          }
        }
      end
      it 'returns status code 200 and the value appears in json' do
        expect(response).to have_http_status(200)
        expect(housing_request.reload.entity_validated).to eq(true)
        expect(housing_request.reload.last_update_user_id).to eq(user2.id.to_s)
      end
    end
    
    describe 'patch request with entity_validated is not valid when done by user' do
      let!(:applicant3) {create(:applicant, user_id: user.id)}
      let!(:housing_request) {create(:housing_request, main_applicant_id: applicant3.id.to_s, user_id: user.id)}
      let!(:user2) {create(:user, auth_roles: ["Entreprise"])}
      let!(:entity) {create(:entity, user_id: user2.id)}
      
      before :each do
        applicant.entity = entity
        applicant.save
        authenticated_header(request, user)
        patch 'update', {
          id: housing_request.id.to_s,
          data: {
            attributes: {entity_validated: true}
          }
        }
      end
      it 'returns status code 200 and the value appears in json' do
        expect(response).to have_http_status(200)
        expect(housing_request.reload.entity_validated).to eq(nil)
      end
    end
    
    describe 'PATCH /housing_requests/:id' do
      housing_request_params = {}
      MUTATE_PARAMS.each do |param, type|
        housing_request_params[param] = get_typed_value(type)
      end
      
      let!(:housing_request) {create(:housing_request, **housing_request_params, main_applicant_id: applicant.id.to_s)}
      MUTATE_PARAMS.each do |param, type|
        describe 'patch request with ' + param.to_s + ' ' + type.to_s + ' is valid' do
          let(:params) do
            attr = {}
            attr[param] = get_typed_value(type, 2)
            attr
          end
          before :each do
            authenticated_header(request, responsible)
            patch 'update', {
              id: housing_request.id.to_s,
              data: {
                attributes: params
              }
            }
          end
          it 'returns status code 200 and the value appears in json' do
            expect(response).to have_http_status(200)
            if (type == 'Date' || type == 'DateTime')
              
              expect(json['data']['attributes'][param.to_s]).not_to be_empty
              expect(json['data']['attributes'][param.to_s]).not_to eq('')
            else
              expect(json['data']['attributes'][param.to_s]).to eq(get_typed_value(type, 2))
            end
          end
        end
      end
      
      describe 'DELETE /housing_requests/:id' do
        let!(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s, user_id: responsible.id.to_s)}
        before :each do
          authenticated_header(request, responsible)
          
          delete 'destroy', {
            id: housing_request.id.to_s
          }
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(204)
        end
      end
    end
    
    describe 'POST /housing_requests/:id/validate_code' do
      let!(:entity1) {create :entity, siret: '00000000000001'}
      let!(:entity2) {create :entity, siret: '00000000000002'}
      let!(:expired_code) {create :entity_code}
      let!(:applicant) {create :applicant}
      let!(:entity_code_with_applicant) {create :entity_code, applicant_id: applicant.id, entity_id: entity1.id, code: 'ABC123'}
      let!(:entity_code_without_applicant) {create :entity_code, entity_id: entity2.id, code: 'DEF456'}
      let(:housing_request) {create :housing_request, main_applicant_id: applicant.id, is_active: true}
      
      before do
        authenticated_header(request, responsible)
      end
      
      it 'requires code parameter' do
        post 'validate_code', {id: housing_request.id.to_s}
        expect(response).to have_http_status 406
      end
      
      it 'returns not_found error' do
        post 'validate_code', {id: housing_request.id.to_s, code: 'unknown_code'}
        expect(response).to have_http_status 404
        expect(json['success']).to eq false
        expect(json['message']).to eq 'Le code n\'est pas valide.'
      end
      
      context 'when entity_code is expired' do
        before do
          expired_code.set(expire_at: Time.zone.now.yesterday)
          post 'validate_code', {id: housing_request.id.to_s, code: expired_code.code}
        end
        it 'returns expired code error' do
          expect(response).to have_http_status 422
          expect(json['success']).to eq false
          expect(json['message']).to eq 'Le code est expiré.'
        end
      end
      
      context 'when entity_code_with_applicant' do
        it 'succeeds' do
          post 'validate_code', {id: housing_request.id.to_s, code: 'ABC123'}
          expect(response).to have_http_status 200
        end
        it 'sets the entity to the applicant' do
          expect do
            post 'validate_code', {id: housing_request.id.to_s, code: 'ABC123'}
          end.to change {applicant.reload.entity_id}.from(nil).to entity1.id
        end
        it 'sets the last_entity_code' do
          expect do
            post 'validate_code', {id: housing_request.id.to_s, code: entity_code_with_applicant.code}
          end.to change {housing_request.reload.last_entity_code}.from(nil).to 'ABC123'
        end
      end
      
      context 'when entity_code_without_applicant' do
        it 'succeeds' do
          post 'validate_code', {id: housing_request.id.to_s, code: 'DEF456'}
          expect(response).to have_http_status 200
        end
        it 'sets the entity to the applicant' do
          expect do
            post 'validate_code', {id: housing_request.id.to_s, code: 'DEF456'}
          end.to change {applicant.reload.entity_id}.from(nil).to entity2.id
        end
        it 'sets the last_entity_code' do
          expect do
            post 'validate_code', {id: housing_request.id.to_s, code: 'DEF456'}
          end.to change {housing_request.reload.last_entity_code}.from(nil).to 'DEF456'
        end
      end

      context 'when code is empty' do
        it 'succeeds' do
          applicant.update(main_employer_siret: entity2.siret)
          expect(housing_request.reload.entity_id).to eq(entity2.id.to_s)
          post 'validate_code', {id: housing_request.id.to_s, code: entity_code_with_applicant.code}
          expect(housing_request.reload.entity_id).to eq(entity1.id.to_s)
          post 'validate_code', {id: housing_request.id.to_s, code: nil}
          expect(response).to have_http_status 200
          expect(housing_request.reload.entity_id).to eq(entity2.id.to_s)
        end
      end

      it 'reload when entity_id is nil' do 
        applicant.update(main_employer_siret: entity1.siret)
        housing_request.update(last_entity_code: entity_code_with_applicant.code)
        housing_request.reload
        applicant.reload
        expect(housing_request.entity_id).to eq(entity1.id.to_s)
        expect(applicant.entity_id).to eq(entity1.id)
        expect(applicant.main_employer_siret).to eq(entity1.siret)
        applicant.update(main_employer_siret: entity2.siret)
        housing_request.reload
        applicant.reload
        expect(housing_request.entity_id).to eq(entity1.id.to_s)
        expect(applicant.entity_id).to eq(entity1.id)
        expect(applicant.main_employer_siret).to eq(entity2.siret)
        post 'validate_code', {id: housing_request.id.to_s, code: nil}
        attributes = json['data']['attributes']
        housing_request.reload
        applicant.reload
        expect(housing_request.entity_id).to eq(entity2.id.to_s)
        expect(attributes['entity_id']).to eq(entity2.id.to_s)
        expect(attributes['main_applicant']['entity_id']).to eq(entity2.id.to_s)
      end 
    end
    
    describe "housing channels management" do
      let!(:channel_1) {create :housing_channel, key: 'dalo'}
      let!(:channel_2) {create :housing_channel, key: 'priority_public'}
      let!(:channel_3) {create :housing_channel, key: 'droits_communs'}
      let!(:channel_4) {create :housing_channel, key: 'key1'}
      let!(:channel_5) {create :housing_channel, key: 'key2'}
      let!(:channel_setting) {create :housing_channel_setting, department: '78', housing_channel_keys: ['dalo', 'priority_public', 'key1', 'key2']}
      
      let(:housing_request) do
        create(
          :housing_request,
          main_applicant: applicant,
          seeked_district_postal_codes: ['78000', '75000'],
          housing_channel_pending_keys: ['dalo'],
          housing_channel_chosen_keys: ['dalo'],
          housing_channel_keys: ['priority_public']
        )
      end
      before do
        authenticated_header(request, responsible)
      end
      
      describe 'POST /housing_requests/:id/add_housing_channel' do
        it 'requires key parameter' do
          post 'add_housing_channel', {id: housing_request.id.to_s}
          expect(response).to have_http_status 406
          expect(json['errors'][0]['message']).to eq 'param is missing or the value is empty: key or keys'
        end
        
        it 'returns not found channel error' do
          post 'add_housing_channel', {id: housing_request.id.to_s, keys: ['unknown']}
          expect(response).to have_http_status 404
          expect(json['errors'][0]['message']).to eq "La filière logement 'unknown' n'existe pas."
        end
        
        xit 'returns already added error' do
          post 'add_housing_channel', {id: housing_request.id.to_s, keys: ['dalo']}
          expect(response).to have_http_status 422
          expect(json['errors'][0]['message']).to eq "La filière logement 'dalo' a déjà été ajoutée."
        end
        
        it 'returns unavailable for department error' do
          post 'add_housing_channel', {id: housing_request.id.to_s, keys: ['droits_communs']}
          expect(response).to have_http_status 422
          expect(json['errors'][0]['message']).to eq "La filière logement 'droits_communs' n'est pas disponible pour les départements '75, 78' recherchés."
        end
        
        xit 'succeeds' do
          expect do
            post 'add_housing_channel', {id: housing_request.id.to_s, keys: ['priority_public']}
          end.to change {housing_request.reload.housing_channel_pending_keys}.from(['dalo']).to(['dalo', 'priority_public'])
                   .and change {housing_request.reload.housing_channel_keys}.from(['priority_public', 'dc']).to ['dc']
        end

        it 'succeeds with keys' do
          expect do
            post 'add_housing_channel', {id: housing_request.id.to_s, keys: ['key1', 'key2']}
          end.to change {housing_request.reload.housing_channel_keys}.from(['priority_public', 'dc']).to(['key1', 'dc', 'key2'])
          expect(response).to have_http_status 200
        end

        it 'succeeds with key' do
          expect do
            post 'add_housing_channel', {id: housing_request.id.to_s, key: 'key1'}
          end.to change {housing_request.reload.housing_channel_keys}.from(['priority_public', 'dc']).to(['priority_public', 'dc', 'key1'])
          expect(response).to have_http_status 200
        end
        
        xit 'sets to false when boolean field exists' do
          expect do
            post 'add_housing_channel', {id: housing_request.id.to_s, keys: ['priority_public']}
          end.to change {housing_request.reload.priority_public}.to false
        end
        
        xit 'adds the key to the housing_channel_chosen_keys' do
          expect do
            post 'add_housing_channel', {id: housing_request.id.to_s, keys: ['priority_public']}
          end.to change {housing_request.reload.housing_channel_chosen_keys}.from(['dalo', 'dc']).to(['dalo', 'dc', 'priority_public'])
        end
        
        #===== TEMPORARY BYPASS VALIDATION =======
        context 'with the temporary bypass' do
          let(:housing_request) do
            create(
              :housing_request,
              main_applicant: applicant,
              seeked_district_postal_codes: ['78000', '75000'],
              housing_channel_chosen_keys: ['dalo'],
              housing_channel_keys: ['dalo']
            )
          end
          
          it 'returns already validated error' do
            post 'add_housing_channel', {id: housing_request.id.to_s, key: 'dalo'}
            expect(response).to have_http_status 422
            expect(json['errors'][0]['message']).to eq "La filière logement 'dalo' a déjà été validée."
          end
          
          it 'succeeds' do
            expect do
              post 'add_housing_channel', {id: housing_request.id.to_s, keys: ['priority_public']}
            end.to change {housing_request.reload.housing_channel_keys}.from(['dalo', 'dc']).to ['priority_public', 'dc']
          end
          it 'succeeds to remove' do
            expect do
              post 'remove_housing_channel', {id: housing_request.id.to_s, keys: ['dalo']}
            end.to change {housing_request.reload.housing_channel_keys}.from(['dalo', 'dc']).to ['dc']
          end
          it 'sets to true when boolean field exists' do
            expect do
              post 'add_housing_channel', {id: housing_request.id.to_s, keys: ['priority_public']}
            end.to change {housing_request.reload.priority_public}.to true
          end
          
          it 'adds the key to the housing_channel_chosen_keys' do
            expect do
              post 'add_housing_channel', {id: housing_request.id.to_s, keys: ['priority_public']}
            end.to change {housing_request.reload.housing_channel_chosen_keys}.from(['dalo']).to(['dalo', 'priority_public'])
          end
        end
        #=========================================
      end
      
      describe 'POST /housing_requests/:id/validate_housing_channel' do
        it 'requires key parameter' do
          post 'validate_housing_channel', {id: housing_request.id.to_s}
          expect(response).to have_http_status 406
          expect(json['errors'][0]['message']).to eq 'param is missing or the value is empty: key or keys'
        end
        
        it 'returns not found channel error' do
          post 'validate_housing_channel', {id: housing_request.id.to_s, keys: ['unknown']}
          expect(response).to have_http_status 404
          expect(json['errors'][0]['message']).to eq "La filière logement 'unknown' n'existe pas."
        end
        
        it 'returns already validated error' do
          post 'validate_housing_channel', {id: housing_request.id.to_s, keys: ['priority_public']}
          expect(response).to have_http_status 422
          expect(json['errors'][0]['message']).to eq "La filière logement 'priority_public' a déjà été validée."
        end
        
        it 'returns not in pending list error' do
          post 'validate_housing_channel', {id: housing_request.id.to_s, keys: ['droits_communs']}
          expect(response).to have_http_status 422
          expect(json['errors'][0]['message']).to eq "La filière logement 'droits_communs' n'est pas dans la liste à valider."
        end
        
        it 'succeeds' do
          expect do
            post 'validate_housing_channel', {id: housing_request.id.to_s, keys: ['dalo']}
          end.to change {housing_request.reload.housing_channel_pending_keys}.from(['dalo']).to([])
                   .and change {housing_request.reload.housing_channel_keys}.from(['priority_public', 'dc']).to ['priority_public', 'dc', 'dalo']
        end
        
        it 'sets to true when boolean field exists' do
          expect do
            post 'validate_housing_channel', {id: housing_request.id.to_s, keys: ['dalo']}
          end.to change {housing_request.reload.dalo}.to true
        end
        
        it 'does not remove the key from housing_channel_chosen_keys' do
          expect(housing_request.housing_channel_pending_keys).to eq ['dalo']
          expect(housing_request.housing_channel_chosen_keys).to eq ['dalo']
          post 'validate_housing_channel', {id: housing_request.id.to_s, keys: ['dalo']}
          
          expect(housing_request.reload.housing_channel_pending_keys).to eq []
          expect(housing_request.reload.housing_channel_chosen_keys).to eq ['dalo']
        end
        
        context 'on sending flux 35' do
          before do
            VCR.use_cassette("request_orientation_task1", :match_requests_on => [:method, :uri, :path]) do
              post 'validate_housing_channel', {id: housing_request.id.to_s, keys: ['dalo']}
            end
          end
          it 'is OK' do
          end
        end
      end
    end
    
    # HOUSING REQUEST FOLDER
    describe 'GET /housing_requests/:id/housing_request_folder' do
      let!(:user1) {create(:user)}
      let!(:user2) {create(:user)}
      let!(:applicant42) {
        create(
          :applicant,
          user: user1,
          monthly_charges: {},
          nationality: "3"
        )
      }
      let!(:applicant2) {
        create(
          :applicant,
          user: user2,
          monthly_charges: {},
          nationality: "2"
        )
      }
      let!(:housing_request5) {
        create(
          :housing_request,
          :with_dependents,
          is_active: true,
          main_applicant: applicant42,
          coapplicants: [{ rel_type: 'P', id:applicant2.id.to_s }],
        )
      }
      describe 'get ok' do
        before :each do
          file = File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'))
          blob = file.read
          RequiredDocument.all.each do |required_doc|
            required_doc.set_blobs([{blob: blob, filename: "minion.jpg"}])
            required_doc.save
          end
          authenticated_header(request, responsible)
          get 'housing_request_folder', {
            id: housing_request5.id.to_s
          }
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json["data"]["attributes"]["required_documents"].length).to eq(7)
          expect(json["data"]["attributes"]["nud_status"]).to eq("Dossier complet prêt pour traitement")
        end
      end
      describe 'get ok' do
        before :each do
          file = File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'))
          blob = file.read
          RequiredDocument.where(type: "Avis d'imposition 2020 sur les revenus de l'année 2019").destroy_all # fix temporaire VCR
          RequiredDocument.all.each do |required_doc|
            required_doc.set_blobs([{blob: blob, filename: "minion.jpg"}])
           
            required_doc.save
            required_doc.start_treatment
            required_doc.save
            required_doc.status = "Pièce validée"
            required_doc.kyc_needed = true
            required_doc.validity_limit = Time.current -  20.days
            required_doc.save
            #ap required_doc
          end
          authenticated_header(request, responsible)
          get 'housing_request_folder', {
            id: housing_request5.id.to_s
          }
        end
        it 'returns a valid json' do
          expect(response).to have_http_status(200)
          expect(json).not_to be_empty
          expect(json["data"]["attributes"]["required_documents"].length).to eq(6)
          expect(json["data"]["attributes"]["required_documents"][0]["status"]).to eq("Date de validité dépassée")
          expect(json["data"]["attributes"]["nud_status"]).to eq("Date de validité dépassée")
        end
      end
      describe 'start_treatment' do
        before :each do
          file = File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'))
          blob = file.read
          
          RequiredDocument.all.each do |required_doc|
            required_doc.set_blobs([{blob: blob, filename: "minion.jpg"}])
            required_doc.save
          end
          authenticated_header(request, responsible)
          post 'start_treatment', {
            id: housing_request5.id.to_s
          }
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
          expect(housing_request5.housing_request_folder.reload.nud_status).to eq("Dossier validé")
        end
      end

      describe 'change_confirmed' do 
        before :each do
          file = File.open([Hermes::Engine.root, %w[spec xml_bouchon minion.jpg]].flatten.join('/'))
          blob = file.read
          
          RequiredDocument.all.each do |required_doc|
            required_doc.set_blobs([{blob: blob, filename: "minion.jpg"}])
            required_doc.save
          end
          authenticated_header(request, responsible)
        end

        it 'error when missing params' do 
          post 'change_confirmed', {
            id: housing_request5.id.to_s
          }
          expect(response).to have_http_status(422)
          expect(housing_request5.housing_request_folder.required_documents.map { |rd| rd.reload.status}.uniq).not_to eq(["Pièce validée"])
        end 

        it 'when all' do 
          post 'change_confirmed', {
            id: housing_request5.id.to_s,
            required_documents:{
              all: true
            }
          }
          expect(response).to have_http_status(200)
          expect(housing_request5.housing_request_folder.required_documents.map { |rd| rd.reload.status}.uniq).to eq(["Pièce validée"])
        end 

        it 'when ids' do 
          hrf = housing_request5.housing_request_folder
          rd1 = hrf.required_documents[0]
          rd2 = hrf.required_documents[1]
          post 'change_confirmed', {
            id: housing_request5.id.to_s,
            required_documents:{
              rd1.id => true,
              rd2.id => false,
            }
          }
          expect(response).to have_http_status(200)
          expect(rd1.reload.status).to eq("Pièce validée")
          expect(rd2.reload.status).to eq("Pièce manquante")
        end 

        
      end 
    end
    
    describe 'GET /housing_requests/:id/housing_request_folder auto replace olds names' do
      let!(:user1) {create(:user)}
      let!(:user2) {create(:user)}
      let!(:applicant42) {
        create(
          :applicant,
          user: user1,
          monthly_charges: {},
          nationality: "3"
        )
      }
     
      let!(:housing_request5) {
        create(
          :housing_request,
          user: user1,
          is_active: true,
          main_applicant: applicant42,
        )
      } 
 
      describe 'get ok' do
        before :each do
          rd = housing_request5.housing_request_folder.required_documents[1]
          rd.type = "Avis d'imposition N-2"
          rd.save
      
          authenticated_header(request, responsible)
          get 'housing_request_folder', {
            id: housing_request5.id.to_s
          }
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json with names replaced' do
          expect(json).not_to be_empty
          expect(json["data"]["attributes"]["required_documents"][1]["type"]).to eq("Avis d'imposition "+ ((Date.current - 1.years).year.to_s )+" sur les revenus de l'année " + ((Date.current - 2.years).year.to_s ))
        
        end
        it 'object is renamed in db if folder recompute' do
          housing_request5.motivations = ["08"]
          housing_request5.save
          rd = housing_request5.reload.housing_request_folder.reload.required_documents[1]
          expect(rd.type).to eq("Avis d'imposition "+ ((Date.current - 1.years).year.to_s )+" sur les revenus de l'année " + ((Date.current - 2.years).year.to_s ))
        
        end
      end
      
    end
    describe 'as reservataire' do
      let!(:reservation_holder1) {create(:reservation_holder)}
      let!(:reservation_holder2) {create(:reservation_holder)}
      let!(:user_applicant) {create(:user, auth_roles: ['demandeur'])}
      let!(:user1) {create(:user, auth_roles: ['reservataire'], reservation_holder_id: reservation_holder1.id)}
      let!(:user2) {create(:user, auth_roles: ['reservataire'], reservation_holder_id: reservation_holder2.id)}
      let!(:user3) {create(:user, auth_roles: ['demandeur'])}
      let!(:main_applicant) {create(:applicant, user: user_applicant)}
      let!(:housing_request1) {create(:housing_request, main_applicant_id: main_applicant.id, reservation_holder_id: reservation_holder1.id)}
      let!(:housing_request2) {create(:housing_request, main_applicant_id: main_applicant.id, reservation_holder_id: reservation_holder2.id)}
      let!(:housing_request3) {create(:housing_request, main_applicant_id: main_applicant.id)}
      let!(:housing_request4) {create(:housing_request, main_applicant_id: main_applicant.id)}
      
      describe 'GET /housing_requests' do
        describe 'access correct request' do
          before :each do
            authenticated_header(request, user1)
            get 'index'
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
      end
      
      describe 'GET /housing_requests/:id' do
        describe 'has access' do
          before :each do
            authenticated_header(request, user1)
            get 'show', {id: housing_request1.id}
          end
          it 'returns results' do
            expect(response.status).to eq(200)
          end
        end
        describe 'is forbidden to access' do
          before :each do
            authenticated_header(request, user1)
            get 'show', {id: housing_request2.id}
          end
          it 'returns results' do
            expect(response.status).to eq(404)
          end
        end
      end
      
      describe 'PATCH /housing_requests/:id' do
        describe 'has access' do
          before :each do
            authenticated_header(request, user1)
            patch 'update', {
              id: housing_request1.id.to_s,
              data: {
                attributes: {
                  contract_type: "test"
                }
              }
            }
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(housing_request1.reload.contract_type).to eq('test')
          end
        end
        
        describe 'is forbidden to access' do
          before :each do
            authenticated_header(request, user1)
            patch 'update', {
              id: housing_request2.id.to_s,
              data: {
                attributes: {
                  contract_type: 'Type'
                }
              }
            }
          end
          it 'returns results' do
            expect(response.status).to eq(404)
          end
        end
      end
      
      describe 'DELETE /housing_requests/:id' do
        describe 'has access' do
          before :each do
            authenticated_header(request, user1)
            delete 'destroy', {
              id: housing_request1.id.to_s
            }
          end
          it 'returns results' do
            expect(response.status).to eq(204)
          end
        end
        describe 'is forbidden to access' do
          before :each do
            authenticated_header(request, user1)
            delete 'destroy', {
              id: housing_request2.id.to_s
            }
          end
          it 'returns results' do
            expect(response.status).to eq(404)
          end
        end
      end
    end
    
    describe 'desactivate and delete' do
      let!(:admin) {create(:user, roles: ['Administrator'])}
      let!(:user_applicant) {create(:user, auth_roles: ['demandeur'])}
      let!(:main_applicant) {create(:applicant, user: user_applicant)}
      let!(:housing_request1) {create(:housing_request, user: admin, main_applicant_id: main_applicant.id, is_active: true)}
      let!(:housing_request2) {create(:housing_request, user: admin, main_applicant_id: main_applicant.id, is_active: true)}
      let!(:housing_request3) {create(:housing_request, user: user_applicant, main_applicant_id: main_applicant.id)}
      let!(:housing_offer1) {create(:housing_offer)}
      let!(:housing_offer2) {create(:housing_offer)}
      let!(:housing_offer3) {create(:housing_offer)}
      let!(:housing_application1) {create(:housing_application, housing_request: housing_request1, housing_offer: housing_offer1)}
      let!(:housing_application2) {create(:housing_application, housing_request: housing_request1, housing_offer: housing_offer2)}
      let!(:housing_application3) {create(:housing_application, housing_request: housing_request3, housing_offer: housing_offer2)}
      
      describe 'PATCH /housing_requests/:id' do
        describe 'on request with applications' do
          before :each do
            authenticated_header(request, admin)
            patch 'update', {
              id: housing_request1.id.to_s,
              data: {
                attributes: {
                  is_active: false
                }
              }
            }
          end
          it 'returns results' do
            expect(housing_request1.housing_offer_ids).to eq([housing_offer1.id.to_s, housing_offer2.id.to_s])
            expect(response.status).to eq(422)
          end
        end
        
        
        describe 'on request without applications' do
          before :each do
            authenticated_header(request, admin)
            patch 'update', {
              id: housing_request2.id.to_s,
              data: {
                attributes: {
                  is_active: false
                }
              }
            }
          end
          it 'returns results' do
            expect(response.status).to eq(200)
          end
        end
      end
      
      describe 'DELETE /housing_requests/:id' do
        describe 'on request with applications' do
          before :each do
            authenticated_header(request, admin)
            delete 'destroy', {
              id: housing_request1.id.to_s
            }
          end
          it 'returns results' do
            expect(response.status).to eq(422)
          end
        end
        describe 'on request without applications' do
          before :each do
            authenticated_header(request, admin)
            delete 'destroy', {
              id: housing_request2.id.to_s
            }
          end
          it 'returns results' do
            expect(response.status).to eq(204)
          end
        end
      end
      
      describe 'unable_to_update_if application' do
        before :each do
          authenticated_header(request, user_applicant)
          patch 'update', {
            id: housing_request3.id.to_s,
            data: {
              attributes: {
                with_car_park: true
              }
            }
          }
        end
        xit 'returns results' do
          expect(response.status).to eq(422)
        end
      end
      describe 'allowed_for_admin' do
        before :each do
          authenticated_header(request, admin)
          patch 'update', {
            id: housing_request3.id.to_s,
            data: {
              attributes: {
                with_car_park: true
              }
            }
          }
        end
        it 'returns results' do
          expect(response.status).to eq(200)
        end
      end
    end
    
    context 'Superadmin searches' do
      let!(:superadmin) {create(:user, auth_roles: ['superadministrateur'])}
      let!(:user) {create(:user)}
      let!(:main_applicant) {
        create(
          :applicant,
          user: superadmin,
          monthly_charges: {},
          nationality: "3"
        )
      }
      
      let!(:applicant_id) {
        create(
          :applicant,
          user: user,
          monthly_charges: {},
          nationality: "2"
        )
      }
      let!(:entity) {create(:entity, title: "test", siret: "12345671234567", id_cristal: "1")}
      
      let!(:housing_request) {
        create(
          :housing_request,
          main_applicant: main_applicant,
          applicant_ids: [applicant_id.id.to_s])
        
      }
    end
    
    context 'Superadmin searches' do
      let!(:superadmin) {create(:user, auth_roles: ['superadministrateur'], first_name: "test_first", last_name: "test_last")}
      let!(:entity1) {create(:entity, title: "test", siret: "00000000000001", id_cristal: "1")}
      let!(:entity2) {create(:entity, title: "test2", siret: "00000000000002", id_cristal: "2")}
      let!(:entity3) {create(:entity, title: "test3", siret: "00000000000003", id_cristal: "3")}
      let!(:applicant1) {create(:applicant, first_name: "John", last_name: "Doe", main_employer_siret: entity1.siret)}
      let!(:applicant2) {create(:applicant, first_name: "Jane", last_name: "Doe", main_employer_siret: entity1.siret)}
      let!(:applicant3) {create(:applicant, first_name: "Bob", last_name: "Dupont", main_employer_siret: entity2.siret)}
      let!(:houing_request1) {create(:housing_request, main_applicant_id: applicant1.id, is_active: true)}
      let!(:houing_request2) {create(:housing_request, main_applicant_id: applicant2.id, is_active: true)}
      let!(:houing_request3) {create(:housing_request, main_applicant_id: applicant3.id, is_active: true)}
      let!(:nud) {create(:nud, applicant_id: applicant2.id, housing_request_id: houing_request2.id )}
      
      describe 'GET /housing_request/admin_search' do
        describe "search with user that is not a Superadmin" do
          before do
            authenticated_header(request, user)
            get 'admin_search'
          end
          
          it "refuses the access" do
            body = JSON.parse(response.body)
            expect(body["code"]).to eq(401)
          end
        end
        
        describe "search with good params" do
          
          before do
            authenticated_header(request, superadmin)
            get 'admin_search', {
              entity_params: {siret: entity1.siret}
            }
          end
          
          it "as valid data" do
            datas = json['data']
            expect(datas.length).to eq(2)
            expect(datas[0]['entity_title']).to eq(entity1.title)
            expect(datas[0]['applicant_first_name']).to eq(applicant1.first_name)
          end
        end
        
        describe "search with good params + user params" do
          
          before do
            authenticated_header(request, superadmin)
            get 'admin_search', {
              entity_params: {siren: entity1.siren},
              applicant_params: {first_name: applicant2.first_name}
            }
          end
          
          it "as valid data" do
            datas = json['data']
            expect(datas.length).to eq(1)
            expect(datas[0]['id']).to eq(houing_request2.id.to_s)
            expect(datas[0]['entity_title']).to eq(entity1.title)
            expect(datas[0]['applicant_first_name']).to eq(applicant2.first_name)
            expect(datas[0]['user_id']).to eq(applicant2.main_user_id.to_s)
          end
        end
        
        describe "search with nud params" do
          
          before do
            authenticated_header(request, superadmin)
            get 'admin_search', {
              unique_numbers: [nud.unique_number]
            }
          end
          
          it "as valid data" do
            datas = json['data']
            expect(datas.length).to eq(1)
            expect(datas[0]['id']).to eq(houing_request2.id.to_s)
            expect(datas[0]['unique_numbers']).to eq([nud.unique_number])
            expect(datas[0]['applicant_first_name']).to eq(applicant2.first_name)
            expect(datas[0]['user_id']).to eq(applicant2.main_user_id.to_s)
          end
        end
      end
    end
    
    context "managing points" do
      describe "POST /housing_requests/:id/add_points_to_housing_request " do
        let!(:user_applicant) {create(:user, auth_roles: ['collaborateur'])}
        let!(:entity) {create(:entity, siret: '00000000000001')}
        let!(:main_applicant) {create(:applicant, user: user_applicant, main_employer_siret: entity.siret)}
        let!(:housing_request) {create(:housing_request, user: user_applicant, main_applicant_id: main_applicant.id, is_active: true)}
        before :each do
          authenticated_header(request, user_applicant)
          entity.applicants << main_applicant
          post "add_points_to_housing_request", {
            id: housing_request.id,
            points: "20",
            entity_id: entity.id
          }
        end
        it "does add points to a housing request" do
          resp = JSON.parse(response.body)
          expect(resp['code']).to eq(200)
        end
      end
      
      describe "GET /housing_requests/:id/waiting_points" do
        let!(:user_applicant) {create(:user, auth_roles: ['collaborateur'])}
        let!(:main_applicant) {create(:applicant, user: user_applicant)}
        let!(:housing_request) {create(:housing_request, user: user_applicant, main_applicant_id: main_applicant.id, is_active: true)}
        let!(:entity) {create(:entity, available_points: 100)}
        let!(:entity_request1) {create(:entity_request_account, amount: 10, entity_id: entity.id)}
        let!(:entity_request2) {create(:entity_request_account, amount: 10, entity_id: entity.id)}
        
        before :each do
          authenticated_header(request, user_applicant)
          housing_request.entity_request_account << entity_request1
          housing_request.entity_request_account << entity_request2
          get "waiting_points", {
            id: housing_request.id
          }
        end
        it "does add the entity request account amounts " do
          resp = JSON.parse(response.body)
          expect(resp['code']).to eq(200)
          expect(resp['message']['waiting_points']).to eq(20)
        end
      end
      
      describe "GET /housing_requests/:id/spent_points" do
        let!(:user_applicant) {create(:user, auth_roles: ['collaborateur'])}
        let!(:main_applicant) {create(:applicant, user: user_applicant)}
        let!(:housing_request) {create(:housing_request, user: user_applicant, main_applicant_id: main_applicant.id, is_active: true)}
        let!(:entity) {create(:entity, available_points: 100)}
        let!(:entity_request1) {create(:entity_request_account, amount: 10, entity_id: entity.id, request_rented: true)}
        let!(:entity_request2) {create(:entity_request_account, amount: 10, entity_id: entity.id, request_rented: true)}
        
        before :each do
          authenticated_header(request, user_applicant)
          housing_request.entity_request_account << entity_request1
          housing_request.entity_request_account << entity_request2
          get "spent_points", {
            id: housing_request.id
          }
        end
        it "does add the entity request account amounts " do
          resp = JSON.parse(response.body)
          expect(resp['code']).to eq(200)
          expect(resp['message']['spent_points']).to eq(20)
        end
      end
    end
    
    context "entity notes and internal notes" do
      describe "GET /housing_requests/:id/entity_note" do
        let!(:user1) {create(:user, auth_roles: ['cre'])}
        let!(:user2) {create(:user)}
        let!(:entity1) {create(:entity, siret: "00000000000001", writers: [user1.id.to_s])}
        let!(:applicant42) {
          create(
            :applicant,
            user: user1,
            monthly_charges: {},
            nationality: "3",
            main_employer_siret: entity1.siret,
          )
        }
        
        let!(:applicant2) {
          create(
            :applicant,
            user: user2,
            monthly_charges: {},
            nationality: "2"
          )
        }
        
        let!(:housing_request5) {
          create(
            :housing_request,
            :with_dependents,
            is_active: true,
            main_applicant: applicant42,
            applicant_ids: [applicant2.id.to_s],
            entity_note: "test note",
            writers: [user1.id.to_s]
          )
        }
        before do
          authenticated_header(request, user1)
          entity1.applicants << applicant42
          get 'entity_note', {
            id: housing_request5.id.to_s
          }
        end
        
        it 'returns the proper entity note' do
          resp = JSON.parse(response.body)
          expect(resp['message']).to eq(housing_request5.entity_note)
        end
      end
      
      describe "GET /housing_requests/:id/internal_note" do
        let!(:user_collab) {create(:user, auth_roles: ['collaborateur offre logement'])}
        let!(:user2) {create(:user)}
        let!(:entity1) {create(:entity, siret: "00000000000001")}
        let!(:applicant42) {
          create(
            :applicant,
            user: user_collab,
            monthly_charges: {},
            nationality: "3",
            main_employer_siret: entity1.siret,
          )
        }
        
        let!(:applicant2) {
          create(
            :applicant,
            user: user2,
            monthly_charges: {},
            nationality: "2"
          )
        }
        
        let!(:housing_request5) {
          create(
            :housing_request,
            :with_dependents,
            is_active: true,
            main_applicant: applicant42,
            applicant_ids: [applicant2.id.to_s],
            internal_note: "test note"
          )
        }
        before do
          authenticated_header(request, user_collab)
          get 'internal_note', {
            id: housing_request5.id.to_s
          }
        end
        
        it 'returns the proper internal note' do
          resp = JSON.parse(response.body)
          expect(resp['message']).to eq(housing_request5.internal_note)
        end
      end
      
      describe "POST /housing_requests/:id/update_internal_note" do
        let!(:user_collab) {create(:user, auth_roles: ['collaborateur offre logement'])}
        let!(:user2) {create(:user)}
        let!(:applicant_collab) {
          create(
            :applicant,
            user: user_collab,
            monthly_charges: {},
            nationality: "3"
          )
        }
        let!(:applicant2) {
          create(
            :applicant,
            user: user2,
            monthly_charges: {},
            nationality: "2",
          )
        }
        let!(:entity1) {create(:entity)}
        
        let!(:housing_request_true) {
          create(
            :housing_request,
            :with_dependents,
            is_active: true,
            main_applicant: applicant_collab,
            applicant_ids: [applicant2.id.to_s],
            internal_note: "test note"
          )
        }
        before do
          authenticated_header(request, user_collab)
        end
        
        it 'returns the proper internal note' do
          entity1.applicants << applicant_collab
          post 'update_internal_note', {
            id: housing_request_true.id.to_s,
            internal_note: "test modified fuck!"
          }
          resp = JSON.parse(response.body)
          expect(resp['message']['value']).to eq("test modified ***")
        end
      end
      
      describe "POST /housing_requests/:id/update_entity_note" do
        let!(:user_collab) {create(:user, auth_roles: ['collaborateur offre logement'])}
        let!(:entity1) {create(:entity, siret: '00000000000001')}
        let!(:user2) {create(:user)}
        let!(:applicant42) {
          create(
            :applicant,
            user: user_collab,
            monthly_charges: {},
            nationality: "3",
            main_employer_siret: entity1.siret
          )
        }
        
        let!(:applicant2) {
          create(
            :applicant,
            user: user2,
            monthly_charges: {},
            nationality: "2"
          )
        }
        
        let!(:housing_request) {
          create(
            :housing_request,
            :with_dependents,
            is_active: true,
            main_applicant: applicant42,
            applicant_ids: [applicant2.id.to_s],
            entity_note: "test note"
          )
        }
        
        before do
          authenticated_header(request, user_collab)
          
          post 'update_entity_note', {
            id: housing_request.id.to_s,
            entity_note: "test modified"
          }
        end
        
        it 'returns the proper internal note' do
          resp = JSON.parse(response.body)
          expect(resp['message']['value']).to eq("test modified")
        end
      end
      
      describe 'POST /housing_requests/:id/notify' do
        let!(:admin) {create(:administrator)}
        let!(:applicant) {create(:applicant)}
        let!(:housing_request) {create(:housing_request, :with_dependents, is_active: true, main_applicant: applicant)}
        
        before do
          authenticated_header(request, admin)
        end
        it 'should send notification' do
          expect {
            post 'notify', id: housing_request.id.to_s, data: {attributes: {title: 'title', body: 'body', channels: ['sms']}}
          }.to change {
            housing_request.notifications.count
          }.by 1
          
          expect(response.status).to eq(201)
          expect(json['data']['id']).not_to be_nil
          expect(json['data']['attributes']['title']).to eq('title')
          expect(json['data']['attributes']['body']).to eq('body')
          expect(json['data']['attributes']['channels']).not_to include('email')
        end
      end
      
      describe 'POST /housing_requests/:id/update_from_sne' do
        let!(:user) {create(:user)}

        let!(:applicant) do
          create(
            :applicant, 
            user: user, 
            nationality: "1", 
            monthly_charges: {}, 
            first_name: "Martine", 
            last_name: "CHOPRON", 
            birth_date: Date.parse('1991-06-07'), 
            auto_set_user: false
          )
        end

        let(:dependents) do
          [
            {"last_name"=>"MOUZOPOULOS", "first_name"=>"Marie", "birth_date"=>"2011-03-07", "rel_type"=>"E", "sex"=>"F"},
            {"last_name"=>"BOUVET", "first_name"=>"Marie", "birth_date"=>"2019-03-20", "rel_type"=>"E", "sex"=>"F"}
          ]
        end

        let!(:housing_request) do
          create(:housing_request, user: user, is_active: true, main_applicant: applicant, dependents: dependents, seeked_localisation_list: [], motivations: ["accueillant", "super"])
        end

        let!(:nud) do
          create(
            :nud, 
            housing_request: housing_request, 
            department: nil, 
            creation_date: '2020-02-02T23:00:00', 
            unique_number: "1110121021492G1104"
          )
        end

        let!(:hri) do 
          create(
            :housing_request_initializer,
            done: false, 
            unique_number: nud.unique_number,
          )

        end 

        before do
          housing_request.reload
          authenticated_header(request, user)
        end

        it 'applicant_verified' do
          post 'update_from_sne', id: housing_request.id.to_s
          old_seeked_district_insee_codes   = housing_request.seeked_district_insee_codes
          old_monthly_revenues              = housing_request.monthly_revenues
          old_seeked_localisation_list      = housing_request.seeked_localisation_list
          housing_request.reload
          new_seeked_district_insee_codes   = housing_request.seeked_district_insee_codes
          new_monthly_revenues              = housing_request.monthly_revenues
          new_seeked_localisation_list      = housing_request.seeked_localisation_list
          
          expect(response.status).to eq(200)
          expect(old_seeked_district_insee_codes).not_to  eq(new_seeked_district_insee_codes)
          expect(old_monthly_revenues).not_to             eq(new_monthly_revenues)
          expect(old_seeked_localisation_list).not_to     eq(new_seeked_localisation_list)
        end

        context 'when not applicant_verified' do 
          before do
            housing_request.reload.update(applicant_verified: false)
            hri.update(hashed_pin_code: hri.hash_string("test"), pin_code_created_at: DateTime.now)
            authenticated_header(request, user)
          end

          it 'error when missing hri' do 
            hri.destroy
            post 'update_from_sne', id: housing_request.id.to_s
            expect(response.status).to eq(404)
          end 

          it 'error when no code' do 
            post 'update_from_sne', id: housing_request.id.to_s
            expect(response.status).to eq(422)
          end 

          it 'error when wrong pin_code' do 
            post 'update_from_sne', id: housing_request.id.to_s, pin_code: 'lol'
            expect(response.status).to eq(422)
          end 

          it 'works' do 
            post 'update_from_sne', id: housing_request.id.to_s, pin_code: 'test'
            expect(response.status).to eq(200)
            expect(housing_request.reload.applicant_verified).to eq(true)
          end 
        end 

        # TODO cassette
        # before do
        #   authenticated_header(request, user)
        #   require 'vcr_config'
        #   VCR.use_cassette("update_from_sne", :match_requests_on => [:method, :uri, :path]) do
        #     housing_request.get_xml_from_sne(nil)
        #   end
        # end

        # it 'should update housing request from sne' do
        #   old_seeked_district_insee_codes   = housing_request.seeked_district_insee_codes
        #   old_monthly_revenues              = housing_request.monthly_revenues
        #   old_seeked_localisation_list      = housing_request.seeked_localisation_list

        #   post 'update_from_sne', id: housing_request.id.to_s

        #   expect(response.status).to eq(200)
        #   housing_request.reload

        #   new_seeked_district_insee_codes   = housing_request.seeked_district_insee_codes
        #   new_monthly_revenues              = housing_request.monthly_revenues
        #   new_seeked_localisation_list      = housing_request.seeked_localisation_list

        #   expect(old_seeked_district_insee_codes).not_to  eq(new_seeked_district_insee_codes)
        #   expect(old_monthly_revenues).not_to             eq(new_monthly_revenues)
        #   expect(old_seeked_localisation_list).not_to     eq(new_seeked_localisation_list)

        #   expect(housing_request.motivation1).to          eq('09')
        #   expect(housing_request.motivation2).to          be_nil
        #   expect(housing_request.motivation3).to          be_nil
        # end
      end

      describe 'POST /housing_requests/send_message' do
        let!(:collaborateur) {create :user, auth_roles: ['collaborateur offre logement']}
        subject {post 'send_message', to: 'test@test.com', data: {attributes: {title: 'title', body: 'body'}}}
        before do
          authenticated_header(request, collaborateur)
        end
        
        it 'creates a new Doc' do
          expect {subject}.to change {Doc.count}.by 1
          
          expect(response.status).to eq(201)
          expect(json['data']['attributes']['title']).to eq('title')
          expect(json['data']['attributes']['body']).to eq('body')
        end
        
        it 'sends an email' do
          expect(Agent).to receive_message_chain(:regular_mail, :deliver_later)
          subject
        end
      end
    end
    
    describe "GET /housing_requests/:id/xml_sopra_errors" do
      let!(:a) {create(:applicant)}
      let!(:hr) {create(:housing_request, main_applicant: a)}
      let!(:hr2) {create(:housing_request, main_applicant: a)}
      let!(:error1) {create(:xml_sopra_error, item_id: hr.id.to_s, item_class: hr.class, error_content: 'test1')}
      let!(:error2) {create(:xml_sopra_error, item_id: hr.id.to_s, item_class: hr.class, error_content: 'test2')}
      let!(:error3) {create(:xml_sopra_error, item_id: hr.id.to_s, item_class: hr.class, error_content: 'test3')}
      let!(:error4) {create(:xml_sopra_error, item_id: hr2.id.to_s, item_class: hr.class, error_content: 'test4')}
      
      before do
        authenticated_header(request, responsible)
        get 'xml_sopra_errors', {id: hr.id.to_s}
      end
      
      it "as valid data" do
        datas = json['data']
        expect(response.status).to eq(200)
        expect(datas.length).to eq(3)
      end
    
    end
  end
  
  require_relative '../../swagger_helper'
  
  describe Api::Dmo::HousingRequestsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_HOUSING_REQUESTS = 'API_DMO_HOUSING_REQUESTS'
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
    let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}
    let!(:applicant) {create(:applicant, user_id: responsible.id.to_s, nationality: "1")}
    let!(:housing_channel) {create :housing_channel, key: 'dalo'}
    let!(:dc_channel) {create(:housing_channel, key: 'dc')}

    
    context 'as superadmin' do
      
      let!(:superadmin) {create(:user, auth_roles: ['superadministrateur'])}
      
      let!(:main_applicant) {
        create(
          :applicant,
          user: superadmin,
          monthly_charges: {},
          nationality: "3"
        )
      }
      
      let!(:applicant_id) {
        create(
          :applicant,
          user: superadmin,
          monthly_charges: {},
          nationality: "2"
        )
      }
      let!(:entity) {create(:entity, title: "test", siret: "12345671234567", id_cristal: "1")}
      
      let!(:housing_request) {
        create(
          :housing_request,
          main_applicant: main_applicant,
          applicant_ids: [applicant_id.id.to_s])
        
      }
      
      path '/api/dmo/housing_requests/admin_search' do
        get 'admin_search ' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          consumes 'application/json'
          
          security [Bearer: {}]
          json_params = {}
          json_params[:id] = {type: :string}
          json_params[:unique_numbers] = {type: :array}
          json_params[:requester_lastname] = {type: :string}
          json_params[:requester_firstname] = {type: :string}
          json_params[:als_id] = {type: :string}
          json_params[:siret] = {type: :string}
          json_params[:id_cristal] = {type: :string}
          json_params[:title] = {type: :string}
          
          parameter name: :params, in: :body, schema: json_params
          
          before do
            entity.applicants << main_applicant
          end
          
          response '200', 'housing_request found' do
            let(:Authorization) {"Bearer #{token_for(superadmin)}"}
            let(:params) do
              {
                id: housing_request.id.to_s
              }
            end
            
            include_context 'with integration test'
            include_context 'not containings errors'
            
            after do |_example|
            end
          end
        end
      end
    end
    
    context 'has a company role' do
      let!(:user_applicant) {create(:user, auth_roles: ['collaborateur'])}
      let!(:entity) {create(:entity, siret: '00000000000001')}
      let!(:main_applicant) {create(:applicant, user: user_applicant, main_employer_siret: entity.siret)}
      let!(:housing_request) {create(:housing_request, user: user_applicant, main_applicant_id: main_applicant.id, is_active: true)}
      
      path '/api/dmo/housing_requests/{id}/add_points_to_housing_request' do
        post 'add points to housing request ' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          
          consumes 'application/json'
          
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          json_params[:points] = {type: :string}
          json_params[:entity_id] = {type: :array}
          
          parameter name: :params, in: :body, schema: json_params
          
          response '200', 'housing_request found' do
            let(:Authorization) {"Bearer #{token_for(user_applicant)}"}
            let(:params) do
              {
                points: "20",
                entity_id: entity.id.to_s
              }
            end
            let(:id) {housing_request.id}
            
            include_context 'with integration test'
            include_context 'not containings errors'
            
            after do |_example|
              json = JSON.parse(response.body)
              expect(json['message']['points_added']).to eq 20
            end
          
          end
        end
      end
      
      path '/api/dmo/housing_requests/{id}/waiting_points' do
        get 'get points that are waiting to be distributed ' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          
          consumes 'application/json'
          
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          
          before do
            entity.applicants << main_applicant
            housing_request.entity_request_account << entity_request1
            housing_request.entity_request_account << entity_request2
          end
          
          response '200', 'housing_request found' do
            let(:Authorization) {"Bearer #{token_for(user_applicant)}"}
            let!(:entity_request1) {create(:entity_request_account, amount: 10, entity_id: entity.id)}
            let!(:entity_request2) {create(:entity_request_account, amount: 10, entity_id: entity.id)}
            let(:id) {housing_request.id}
            
            include_context 'with integration test'
            include_context 'not containings errors'
            
            after do |_example|
              json = JSON.parse(response.body)
              expect(json['message']['waiting_points']).to eq 20
            end
          
          end
        end
      end
      
      path '/api/dmo/housing_requests/{id}/spent_points' do
        get 'get points that are waiting to be distributed ' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          
          consumes 'application/json'
          
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          
          before do
            entity.applicants << main_applicant
            housing_request.entity_request_account << entity_request1
            housing_request.entity_request_account << entity_request2
          end
          
          response '200', 'housing_request found' do
            let(:Authorization) {"Bearer #{token_for(user_applicant)}"}
            let!(:entity_request1) {create(:entity_request_account, amount: 10, entity_id: entity.id, request_rented: true)}
            let!(:entity_request2) {create(:entity_request_account, amount: 10, entity_id: entity.id, request_rented: true)}
            
            let(:id) {housing_request.id}
            
            include_context 'with integration test'
            include_context 'not containings errors'
            
            after do |_example|
              json = JSON.parse(response.body)
              expect(json['message']['spent_points']).to eq 20
            end
          
          end
        end
      end
    end
    
    context 'as admin' do
      let!(:admin) {create(:administrator)}
      let!(:housing_channel) {create :housing_channel, key: 'dalo'}
      path '/api/dmo/housing_requests' do
        post 'Create housing_request' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          json_params[:main_applicant_id] = {type: :string}
          json_params[:coapplicants] = {type: :array}
          json_params[:dependents] = {type: :array}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)
          
          response '201', 'housing_request created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:params) do
              jsonapi_attributes(
                {main_applicant_id: applicant.id.to_s}
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            
            after do |_example|
              json = JSON.parse(response.body)
              
              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_requests'
            end
          end
        end
      end
      
      path '/api/dmo/housing_requests' do
        get 'Retrieves all housing_requests' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          
          parameter name: 'ids[]', in: :query, type: :array, items: {type: :string}, collectionFormat: :multi, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = :string
            if MUTATE_PARAMS[acceptable_param]
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end
          # VALID_OPERATORS.each do |valid_operator|
          #    parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
          #  end
          
          security [Bearer: {}]
          paginator
          
          response '200', 'housing_requests found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let!(:housing_requests) {create_list(:housing_request, 5, main_applicant_id: applicant.id.to_s)}
            
            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) {housing_requests}
            end
          end
        end
      end
      
      path '/api/dmo/housing_requests/{id}' do
        get 'Retrieves a single housing_request by id' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          
          response '200', 'housing_request found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s)}
            let(:id) {housing_request.id}
            
            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) {housing_request}
            end
          end
        end
      end
      
      # path '/api/dmo/housing_requests/clp_details' do
      #   get 'Retrieves a single housing_request by id' do
      #     tags TAGS_API_DMO_HOUSING_REQUESTS
      #     produces 'application/json'
      #     parameter name: 'als_ids[]', in: :query, type: :array, items: {type: :string}, required: true, collectionFormat: :multi
      #     ACCEPTABLE_PARAMS.each do |acceptable_param|
      #       type = :string
      #       if MUTATE_PARAMS[acceptable_param]
      #         type = nice_type(MUTATE_PARAMS[acceptable_param])
      #       end
      #       parameter name: acceptable_param.to_s, in: :query, type: type, required: false
      #     end
      #     security [Bearer: {}]
      #     paginator
      #     response '200', 'housing_request found' do
      #       let(:Authorization) {"Bearer #{token_for(admin)}"}
      #       let(:housing_requests) {create_list(:housing_request, 5, main_applicant_id: applicant.id.to_s)}
      #       let(:requests_sample) {2.times.map {|i| housing_requests[i]}}
      #       # drain_all_jobs
      #       let!(:'als_ids[]') {requests_sample.map(&:als_id)}
            
      #       include_context 'with integration test'
      #       include_context 'valid jsonapi collection' do
      #         let(:expected_collection) {requests_sample}
      #       end
      #     end
      #   end
      # end
      
      path '/api/dmo/housing_requests/{id}/housing_request_folder' do
        get 'Retrieves the housing_request_folder of an housing_request' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string, required: true
          security [Bearer: {}]
          response '200', 'housing_request_folder found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s)}
            let(:id) {housing_request.id}
            
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            after do |_example|
              JSON.parse(response.body)
            end
          end
        end
      end
      
      path '/api/dmo/housing_requests/{id}/add_housing_channel' do
        post 'Add a new key to the housing request housing_channel_pending_keys array' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string, required: true
          parameter name: :params, in: :body, schema: {
            type: :object,
            properties: {
              keys: { type: :array, items: { type: :string }, collectionFormat: :multi },
              required: [:keys]
            }
          }
          security [Bearer: {}]
          response '200', 'housing channel key added' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s)}
            let(:id) {housing_request.id}
            let(:params) do
              {keys: ['dalo']}
            end
            
            include_context 'with integration test'
            include_context 'not containings errors'
            after do |_example|
              JSON.parse(response.body)
            end
          end
        end
      end
      
      path '/api/dmo/housing_requests/{id}/validate_housing_channel' do
        post 'Validate a pending key to the housing request housing_channel_keys array' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string, required: true
          parameter name: :params, in: :body, schema: {
            type: :object,
            properties: {
              keys: { type: :array, items: { type: :string }, collectionFormat: :multi },
              required: [:keys]
            }
          }
          security [Bearer: {}]
          response '200', 'housing channel key validated' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s, housing_channel_pending_keys: ['dalo'])}
            let(:id) {housing_request.id}
            let(:params) do
              {keys: ['dalo']}
            end
            
            include_context 'with integration test'
            include_context 'not containings errors'
            after do |_example|
              JSON.parse(response.body)
            end
          end
        end
      end
      
      path '/api/dmo/housing_requests/{id}/validate_code' do
        post 'Validate an entity_code for the main applicant' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string, required: true
          parameter name: :params, in: :body, schema: {
            type: :object,
            properties: {
              code: {type: :string},
              required: [:code]
            }
          }
          security [Bearer: {}]
          response '200', 'entity code validated for the main applicant' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:entity) {create :entity}
            let(:entity_code) {create :entity_code, entity_id: entity.id, applicant_id: applicant.id}
            let(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s)}
            let(:id) {housing_request.id}
            let(:params) do
              {code: entity_code.code}
            end
            
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            after do |_example|
              JSON.parse(response.body)
            end
          end
        end
      end

      # TODO fix

      # path '/api/dmo/housing_requests/{id}/update_from_sne' do
      #   require 'vcr_config'
      #   post 'update the info with ressources on SNE' do
      #     tags TAGS_API_DMO_HOUSING_REQUESTS
      #     produces 'application/json'
      #     consumes 'application/json'
      #     parameter name: :id, in: :path, type: :string, required: true
      #     security [Bearer: {}]
      #     response '200', 'info have been updated' do
            
      #       let!(:user1) {create(:user)}
      #       let(:Authorization) {"Bearer #{token_for(user1)}"}
      #       let!(:applicant1) do
      #         create(
      #           :applicant,
      #           user: user1,
      #           nationality: "1",
      #           monthly_charges: {},
      #           first_name: "JOSSELINE",
      #           last_name: "LIBAR",
      #           birth_date: Date.parse("1954-05-06")
      #         )
      #       end
            
      #       let!(:xml) do 
      #         create(
      #           :xml,
      #           xml: "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><ns1:interfaceNuu xmlns:ns1=\"http://nuu.application.i2/\" versionInterface=\"03.00\"><ns1:entete><ns1:typeFichier>DIS</ns1:typeFichier><ns1:dateFichier>2020-04-07T11:51:39</ns1:dateFichier></ns1:entete><ns1:demande><ns1:demandeLogement><ns1:numGuichet>G1175</ns1:numGuichet><ns1:numUnique>1110220019793G1175</ns1:numUnique><ns1:dateCreationDemande>2020-02-24</ns1:dateCreationDemande><ns1:forcerDoublon>false</ns1:forcerDoublon><ns1:renouvellementElec>false</ns1:renouvellementElec><ns1:precision></ns1:precision><ns1:ressourcesLoiEC>9600</ns1:ressourcesLoiEC><ns1:informationTraitement><ns1:estDelaiAnormalementLong>false</ns1:estDelaiAnormalementLong><ns1:estMultiplesAttributionsSousConditions>false</ns1:estMultiplesAttributionsSousConditions><ns1:dateCalculIndicateur>2020-04-05T00:00:00</ns1:dateCalculIndicateur></ns1:informationTraitement><ns1:logementRecherche><ns1:categorieLogementRecherche code=\"I\"/><ns1:avecParking>false</ns1:avecParking><ns1:rezDeChaussee>false</ns1:rezDeChaussee><ns1:sansAscenseur>false</ns1:sansAscenseur><ns1:demandeElargie>false</ns1:demandeElargie><ns1:listeTypeLogement><ns1:typeLogement code=\"T3\"/></ns1:listeTypeLogement><ns1:listeLocalisationSouhaite><ns1:localisationSouhaite><ns1:commune code=\"77349\"/><ns1:codePostal/><ns1:seuilEPCIPremierQuartile>9692</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>1</ns1:quartileAppartenance></ns1:localisationSouhaite></ns1:listeLocalisationSouhaite></ns1:logementRecherche><ns1:principauxMotifDemande><ns1:motifDemande code=\"01\"/></ns1:principauxMotifDemande><ns1:personnePhysique><ns1:demandeur><ns1:civilite code=\"2\"/><ns1:nom>LIBAR</ns1:nom><ns1:prenom>JOSSELINE</ns1:prenom><ns1:dateNaissance>1954-05-06</ns1:dateNaissance><ns1:nationalite code=\"1\"/><ns1:situationFamiliale code=\"C\"/><ns1:telPortable>0613498988</ns1:telPortable><ns1:notifSms>true</ns1:notifSms><ns1:notifEvt>false</ns1:notifEvt><ns1:adressePostale><ns1:etranger>false</ns1:etranger><ns1:numero>07</ns1:numero><ns1:voie>PLACE ILE DE RANCE</ns1:voie><ns1:codePostal>93150</ns1:codePostal><ns1:commune code=\"93007\"/></ns1:adressePostale><ns1:situationProfessionnelle><ns1:profession>VENDEUSE</ns1:profession><ns1:typeContratTravail code=\"CDIP\"/><ns1:nomEmployeurPrincipal>ZARA</ns1:nomEmployeurPrincipal><ns1:etranger>false</ns1:etranger><ns1:codePostal>75016</ns1:codePostal><ns1:commune code=\"75116\"/></ns1:situationProfessionnelle><ns1:revenuFiscal><ns1:anneeMoins2>2018</ns1:anneeMoins2><ns1:montantMoins2>5326</ns1:montantMoins2></ns1:revenuFiscal><ns1:listeRessourceRecue><ns1:detailRessource><ns1:ressource code=\"SAL\"/><ns1:montant>1200</ns1:montant></ns1:detailRessource></ns1:listeRessourceRecue><ns1:listeRessourceVersee/><ns1:listeRessourceRecuePersACharge/><ns1:listeRessourceVerseePersACharge/></ns1:demandeur><ns1:listePersonneACharge><ns1:personneACharge><ns1:nom>LIBAR</ns1:nom><ns1:prenom>KILLIANE</ns1:prenom><ns1:dateNaissance>1999-04-18</ns1:dateNaissance><ns1:sexe code=\"F\"/><ns1:lienParente code=\"E\"/></ns1:personneACharge></ns1:listePersonneACharge><ns1:logementActuel><ns1:listeModeleLogement><ns1:modeleLogement code=\"HCPE\"/></ns1:listeModeleLogement><ns1:montantMensuelLoyer>0</ns1:montantMensuelLoyer><ns1:montantMensuelAPL>0</ns1:montantMensuelAPL><ns1:nombreHabitant>6</ns1:nombreHabitant><ns1:typeLogementActuel code=\"T3\"/><ns1:surface>54</ns1:surface><ns1:proprietaire>false</ns1:proprietaire><ns1:etranger>false</ns1:etranger></ns1:logementActuel></ns1:personnePhysique></ns1:demandeLogement></ns1:demande></ns1:interfaceNuu>"
      #         )
      #       end
      #       let!(:housing_request1) do
      #         create(
      #           :housing_request,
      #           is_active: true,
      #           main_applicant: applicant1,
      #           unborn_children: [{
      #                               'anticipated_birth' => '2020-03-23T00:00:00.000Z'
      #                             }],
      #           seeked_localisation_list: [
      #           ],
      #           is_sne_update_available: true,
      #           xml_res: "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><ns1:interfaceNuu xmlns:ns1=\"http://nuu.application.i2/\" versionInterface=\"03.00\"><ns1:entete><ns1:typeFichier>DIS</ns1:typeFichier><ns1:dateFichier>2020-04-07T11:51:39</ns1:dateFichier></ns1:entete><ns1:demande><ns1:demandeLogement><ns1:numGuichet>G1175</ns1:numGuichet><ns1:numUnique>1110220019793G1175</ns1:numUnique><ns1:dateCreationDemande>2020-02-24</ns1:dateCreationDemande><ns1:forcerDoublon>false</ns1:forcerDoublon><ns1:renouvellementElec>false</ns1:renouvellementElec><ns1:precision></ns1:precision><ns1:ressourcesLoiEC>9600</ns1:ressourcesLoiEC><ns1:informationTraitement><ns1:estDelaiAnormalementLong>false</ns1:estDelaiAnormalementLong><ns1:estMultiplesAttributionsSousConditions>false</ns1:estMultiplesAttributionsSousConditions><ns1:dateCalculIndicateur>2020-04-05T00:00:00</ns1:dateCalculIndicateur></ns1:informationTraitement><ns1:logementRecherche><ns1:categorieLogementRecherche code=\"I\"/><ns1:avecParking>false</ns1:avecParking><ns1:rezDeChaussee>false</ns1:rezDeChaussee><ns1:sansAscenseur>false</ns1:sansAscenseur><ns1:demandeElargie>false</ns1:demandeElargie><ns1:listeTypeLogement><ns1:typeLogement code=\"T3\"/></ns1:listeTypeLogement><ns1:listeLocalisationSouhaite><ns1:localisationSouhaite><ns1:commune code=\"77349\"/><ns1:codePostal/><ns1:seuilEPCIPremierQuartile>9692</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>1</ns1:quartileAppartenance></ns1:localisationSouhaite></ns1:listeLocalisationSouhaite></ns1:logementRecherche><ns1:principauxMotifDemande><ns1:motifDemande code=\"01\"/></ns1:principauxMotifDemande><ns1:personnePhysique><ns1:demandeur><ns1:civilite code=\"2\"/><ns1:nom>LIBAR</ns1:nom><ns1:prenom>JOSSELINE</ns1:prenom><ns1:dateNaissance>1954-05-06</ns1:dateNaissance><ns1:nationalite code=\"1\"/><ns1:situationFamiliale code=\"C\"/><ns1:telPortable>0613498988</ns1:telPortable><ns1:notifSms>true</ns1:notifSms><ns1:notifEvt>false</ns1:notifEvt><ns1:adressePostale><ns1:etranger>false</ns1:etranger><ns1:numero>07</ns1:numero><ns1:voie>PLACE ILE DE RANCE</ns1:voie><ns1:codePostal>93150</ns1:codePostal><ns1:commune code=\"93007\"/></ns1:adressePostale><ns1:situationProfessionnelle><ns1:profession>VENDEUSE</ns1:profession><ns1:typeContratTravail code=\"CDIP\"/><ns1:nomEmployeurPrincipal>ZARA</ns1:nomEmployeurPrincipal><ns1:etranger>false</ns1:etranger><ns1:codePostal>75016</ns1:codePostal><ns1:commune code=\"75116\"/></ns1:situationProfessionnelle><ns1:revenuFiscal><ns1:anneeMoins2>2018</ns1:anneeMoins2><ns1:montantMoins2>5326</ns1:montantMoins2></ns1:revenuFiscal><ns1:listeRessourceRecue><ns1:detailRessource><ns1:ressource code=\"SAL\"/><ns1:montant>1200</ns1:montant></ns1:detailRessource></ns1:listeRessourceRecue><ns1:listeRessourceVersee/><ns1:listeRessourceRecuePersACharge/><ns1:listeRessourceVerseePersACharge/></ns1:demandeur><ns1:listePersonneACharge><ns1:personneACharge><ns1:nom>LIBAR</ns1:nom><ns1:prenom>KILLIANE</ns1:prenom><ns1:dateNaissance>1999-04-18</ns1:dateNaissance><ns1:sexe code=\"F\"/><ns1:lienParente code=\"E\"/></ns1:personneACharge></ns1:listePersonneACharge><ns1:logementActuel><ns1:listeModeleLogement><ns1:modeleLogement code=\"HCPE\"/></ns1:listeModeleLogement><ns1:montantMensuelLoyer>0</ns1:montantMensuelLoyer><ns1:montantMensuelAPL>0</ns1:montantMensuelAPL><ns1:nombreHabitant>6</ns1:nombreHabitant><ns1:typeLogementActuel code=\"T3\"/><ns1:surface>54</ns1:surface><ns1:proprietaire>false</ns1:proprietaire><ns1:etranger>false</ns1:etranger></ns1:logementActuel></ns1:personnePhysique></ns1:demandeLogement></ns1:demande></ns1:interfaceNuu>"
      #         )
      #       end
      #       let(:id) {housing_request1.id}
      #       let!(:nud1) do
      #         create(
      #           :nud,
      #           housing_request: housing_request1,
      #           department: '12',
      #           creation_date: '2016-23-14T14:53:22',
      #           unique_number: "1110220019793G1175"
      #         )
      #       end
            
      #       include_context 'with integration test'
      #       include_context 'not containings errors'
      #       after do |_example|
              
      #         json = JSON.parse(response.body)
      #       end
      #     end
      #   end
      # end
      path '/api/dmo/housing_requests/{id}' do
        put 'edit housing_request' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          json_params[:main_applicant_id] = {type: :string}
          json_params[:coapplicants] = {type: :array}
          json_params[:reference_tax_revenues] = {type: :float}
          json_params[:dependents] = {type: :array}
          
          parameter name: :params, in: :body, schema: jsonify_params(json_params)
          #jsonapi_params( **json_params )
          
          response '200', 'housing_request updated' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:housing_request) {create(:housing_request, main_applicant_id: applicant.id.to_s)}
            let(:id) {housing_request.id}
            let(:params) do
              jsonapi_attributes(
                {
                  attributed_housing_district: "RETENUE EN CAL"
                }
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'
            
            after do |_example|
              json = JSON.parse(response.body)
              
              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'housing_requests'
            end
          end
        end
      end
      
      path '/api/dmo/housing_requests/{id}' do
        delete 'Delete a single housing_request by id' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          
          response '204', 'housing_request deleted' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:id) {create(:housing_request, user_id: responsible.id.to_s, main_applicant_id: applicant.id.to_s).id}
            run_test!
          end
        end
      end
    end
    
    context 'as a collaborator' do
      let!(:user_ent) {create(:user, auth_roles: ['cre'])}
      let!(:user_collab) {create(:user, auth_roles: ['collaborateur offre logement'])}
      let!(:entity) {create(:entity, writers: [user_ent.id.to_s], siret: '00000000000001')}
      let!(:main_applicant) {
        create(
          :applicant,
          user: user_ent,
          monthly_charges: {},
          nationality: "3",
          main_employer_siret: entity.siret
        )
      }
      let!(:applicant) {
        create(
          :applicant,
          user: user_ent,
          monthly_charges: {},
          nationality: "2"
        )
      }
      let!(:housing_request) {
        create(
          :housing_request,
          :with_dependents,
          is_active: true,
          main_applicant: main_applicant,
          applicant_ids: [applicant.id.to_s],
          entity_note: "test note",
          internal_note: "test note"
        )
      }
      
      path '/api/dmo/housing_requests/{id}/update_entity_note' do
        put 'Create / Update entity note' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          
          json_params = {}
          json_params[:entity_note] = {type: :string}
          parameter name: :id, in: :path, type: :string
          
          parameter name: :params, in: :body, schema: json_params
          
          response '200', 'housing_request modified' do
            let(:Authorization) {"Bearer #{token_for(user_ent)}"}
            let(:id) {housing_request.id.to_s}
            let(:params) do
              {entity_note: "test modified"}
            end
            
            include_context 'with integration test' # run le test
            include_context 'not containings errors'
            after do |_example|
              json = JSON.parse(response.body)
              expect(json['message']['value']).to eq("test modified")
            end
          end
        end
      end
      
      path '/api/dmo/housing_requests/{id}/update_internal_note' do
        put 'Create / Update internal note' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          
          json_params = {}
          json_params[:internal_note] = {type: :string}
          parameter name: :id, in: :path, type: :string
          
          parameter name: :params, in: :body, schema: json_params
          before do
            entity.applicants << main_applicant
          end
          
          response '200', 'housing_request modified' do
            let(:Authorization) {"Bearer #{token_for(user_collab)}"}
            let(:id) {housing_request.id.to_s}
            let(:params) do
              {internal_note: "test modified fuck"}
            end
            
            include_context 'with integration test' # run le test
            include_context 'not containings errors'
            after do |_example|
              json = JSON.parse(response.body)
              expect(json['message']['value']).to eq("test modified ***")
            end
          end
        end
      end
      
      path '/api/dmo/housing_requests/{id}/internal_note' do
        get 'Get internal note' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          
          parameter name: :id, in: :path, type: :string
          
          response '200', 'housing_request modified' do
            let(:Authorization) {"Bearer #{token_for(user_collab)}"}
            let(:id) {housing_request.id.to_s}
            
            include_context 'with integration test' # run le test
            include_context 'not containings errors'
            after do |_example|
              json = JSON.parse(response.body)
              expect(json['message']).to eq("test note")
            end
          end
        end
      end
      
      path '/api/dmo/housing_requests/{id}/entity_note' do
        get 'Get internal note' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          
          parameter name: :id, in: :path, type: :string
          
          before do
            entity.applicants << main_applicant
            housing_request.writers = [user_ent.id.to_s]
            housing_request.save
          end
          response '200', 'housing_request modified' do
            let(:Authorization) {"Bearer #{token_for(user_ent)}"}
            let(:id) {housing_request.id.to_s}
            
            include_context 'with integration test' # run le test
            include_context 'not containings errors'
            after do |_example|
              json = JSON.parse(response.body)
              expect(json['message']).to eq("test note")
            end
          end
        end
      end
      
      path '/api/dmo/housing_requests/{id}/notify' do
        post 'Send custom notification to main applicant' do
          tags TAGS_API_DMO_HOUSING_REQUESTS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          
          json_params = {}
          json_params[:title] = {type: :string}
          json_params[:body] = {type: :string}
          json_params[:channels] = {type: :array}
          parameter name: :id, in: :path, type: :string
          
          parameter name: :params, in: :body, schema: jsonify_params(json_params)
          
          response '201', 'notifiication created' do
            let(:Authorization) {"Bearer #{token_for(user_collab)}"}
            let(:id) {housing_request.id.to_s}
            let(:params) do
              jsonapi_attributes({
                                   title: 'title',
                                   body: 'body',
                                   channels: ['sms']
                                 })
            end
            
            include_context 'with integration test' # run le test
            include_context 'not containings errors'
            after do |_example|
              json = JSON.parse(response.body)
              expect(json['data']['id']).not_to be_nil
              expect(json['data']['attributes']['title']).to eq('title')
              expect(json['data']['attributes']['body']).to eq('body')
              expect(json['data']['attributes']['channels']).not_to include('email')
            end
          end
        end
      end
    end
  end
end
