# frozen_string_literal: true

require 'rails_helper'

# %w[
#   applicant
#   housing_request
# ].each { |factory_name|require Hermes::Engine.root.join('spec', 'factories', "#{factory_name.pluralize}_factory").to_s }

RSpec.describe Hermes::HousingRequest, type: :model do
  let!(:user) {create(:user)}
  describe 'basic controls' do
    let!(:user1) {create(:user)}
    let!(:applicant1) do
      create(
        :applicant,
        user: user1,
        monthly_charges: {}
      )
    end
    
    it 'has a valid factory' do
      expect(FactoryBot.build(:housing_request, main_applicant: applicant1)).to be_valid
    end
  end
  
  context 'with simple setup' do
    before :each do
      $redis.flushall
    end
    let!(:user1) {create(:user)}
    let!(:user2) {create(:user)}
    let!(:user3) {create(:user)}
    let!(:applicant1) {
      create(
        :applicant,
        user: user1,
        monthly_charges: {},
      )
    }
    let!(:applicant2) {
      create(
        :applicant,
        user: user2
      )
    }
    let!(:applicant3) {
      create(
        :applicant,
        user: user3
      )
    }
    
    let!(:housing_request1) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: applicant1
      )
    end
    
    let!(:housing_request2) do
      create(
        :housing_request,
        is_active: true,
        main_applicant: applicant1,
        coapplicants: [
          {rel_type: 'Conjoint (e)', id: applicant2.id.to_s},
          {rel_type: 'Parent (e)', id: applicant3.id.to_s}
        ]
      )
    end
    
    describe 'Eligibility query are valid' do
      it 'should include right typology ranges' do
        typology_range1 = housing_request1.update_typology_range
        typology_range2 = housing_request2.update_typology_range
        range1_to_i = typology_range1.map {|typology| Integer(typology.sub('T', ''))}
        range2_to_i = typology_range2.map {|typology| Integer(typology.sub('T', ''))}
        
        (1..2).each do |n|
          (1..2).each do |i|
            expect(binding.local_variable_get("range#{n}_to_i".to_sym)).to include i
          end
        end
        expect(range1_to_i.max).to be 2
        expect(range2_to_i).to include 3
        expect(range2_to_i.max).to be 3
      end
      
      it 'eligibility should incorpore typology' do
        housing_request1.update_typology_range
        typology_filter1 = housing_request1.typology_filter
        expect(typology_filter1[:typology][:$in]).to include 'T1'
        expect(typology_filter1[:typology][:$in]).to include 'T2'
      end
      
      describe 'edit action dont duplicate coapplicants' do
        before :each do
          housing_request2.coapplicants = [
            {rel_type: 'Conjoint (e)', id: applicant3.id}
          ]
          housing_request2.seeking_elevator = false
          housing_request2.save
          housing_request2.reload
          housing_request2.coapplicants = [{rel_type: 'M', id: applicant3.id.to_s}]
          housing_request2.save
          applicant3.destroy
          housing_request2.reload
          housing_request2.forcing_duplicate = true
          housing_request2.save
          housing_request2.reload
        end
        it 'coapplicants doesnt change' do
          expect(housing_request2.reload.applicant_ids.length).to eq(0)
        end
      end
    end
    
    describe "#update_seeked_location" do
      before :each do
      $redis.flushall
      end
      let(:list) do
        {'nom': 'Fréjus', 'code': '83061', 'codeDepartement': '81', 'codesPostaux': ['83900', '83370', '']}
      end
      let(:lists) do
        [
          {'nom': 'Paris', 'code': '75001', 'codeDepartement': '75', 'codesPostaux': ['75006', nil]},
          {'nom': 'Meaux', 'code': '77000', 'codeDepartement': '77'},
          {'nom': 'Paris', 'code': '75002', 'codeDepartement': '75', 'codesPostaux': ['75006', '93000']}
        ]
      end
      let(:list_without_postal_codes) do
        [
          {'nom': 'Meaux', 'code': '77000', 'codeDepartement': '77'},
        ]
      end

      let(:broken_list) do
        {'codePostal': '77013', 'foo': 'bir'}
      end
      
      it 'sets seeked fields on save' do
        housing_request1.seeked_localisation_list = [list]
        expect {housing_request1.save}.to change {housing_request1.seeked_departments}.from([]).to(['81'])
                                            .and change {housing_request1.truly_seeked_departments}.from([]).to(['81', '83'])
                                                   .and change {housing_request1.seeked_district_postal_codes}.from([]).to(['83370', '83900'])
                                                          .and change {housing_request1.seeked_districts}.from([]).to(['Fréjus'])
      end
      
      it 'ensures the sorted uniqueness of seeked fields' do
        housing_request1.seeked_localisation_list = lists + [list]
        housing_request1.save
        expect(housing_request1.seeked_departments).to eq ['75', '77', '81']
        expect(housing_request1.truly_seeked_departments).to eq ['75', '77', '81', '83', '93']
        expect(housing_request1.seeked_district_postal_codes).to eq ['75006', '83370', '83900', '93000']
        expect(housing_request1.seeked_districts).to eq ['Fréjus', 'Meaux', 'Paris']
      end
      
      it 'get truly_seeked_departments from insee code' do
        housing_request1.seeked_localisation_list = list_without_postal_codes
        expect {housing_request1.save}.to change {housing_request1.truly_seeked_departments}.from([]).to(['77'])
      end

      it 'returns error' do
        housing_request1.seeked_localisation_list = [broken_list]
        housing_request1.save
        expect(housing_request1.errors.full_messages.first).to match /non-conform seeked_localisation_list/
      end
    end
    
    describe 'Relationships are valid' do
      before :each do
        $redis.flushall
      end
      it 'should has correct housing_requests situation' do
        expect(applicant1.first_active_request.id).to eq(housing_request2.id)
        expect(applicant1.housing_requests.length).to eq(2)
        expect(applicant1.housing_requests.actives.length).to eq(1)
      end
      
      it 'should has correct values' do
        revenues_sum = (applicant1.monthly_revenues || 0) + (applicant2.monthly_revenues || 0)
        expect(housing_request2.monthly_revenues).to eq(revenues_sum)
      end
      
      it 'should has correct coapplicants' do
        coapplicants = housing_request2.coapplicants
        expect(coapplicants.length).to eq(2)
        expect(coapplicants.map {|coapplicant| coapplicant[:rel_type]}).to include('Conjoint (e)').and include('Parent (e)')
        expect(coapplicants.map {|coapplicant| coapplicant[:id]}).to include(applicant2.id).and include(applicant3.id)
      end
      
      describe 'after coapplicants update' do
        before :each do
          housing_request2.coapplicants = [
            {rel_type: 'Conjoint (e)', id: applicant3.id}
          ]
          housing_request2.save
        end
        
        it 'should has updated with valid data' do
          r_request = housing_request2.reload
          expect(r_request.applicant_ids.length).to be 1
          expect(r_request.applicant_ids.first).to eq applicant3.id
          expect(r_request.coapplicants.first[:rel_type]).to eq 'Conjoint (e)'
        end
      end
    end
    
    describe 'applicant should have only public offer as eligibility when last active request becomes inactive' do
      it 'should has first active request with valid data' do
        expect(housing_request2.main_applicant.housing_requests.actives.count).to eq 1
        expect(housing_request2.is_active).to be true
        expect(housing_request2.main_applicant.first_active_request.id.to_s).to eq housing_request2.id.to_s
      end
      
      context 'is_active is set to false' do
        before :each do
          housing_request2.update(is_active: false)
        end
        
        it 'should has is_active == false' do
          expect(housing_request2.reload.is_active).to be false
        end
        
        it 'should has its main_applicant.eligibility == {is_public: true}' do
          expect(housing_request2.reload.main_applicant.eligibility_query).to eq '{"is_public":true}'
        end
      end
    end
    
  end

  describe "2 active housing request with same nud is forbidden" do
    let!(:applicant) {create(:applicant)}
    let!(:applicant2) {create(:applicant)}
    let!(:request1) {create(:housing_request, main_applicant: applicant, is_active: true)}
    let!(:request2) {create(:housing_request, main_applicant: applicant2, is_active: false)}
    let!(:nud_test) {create(:nud, housing_request: request1, unique_number: "011752019061716361")}
    let!(:nud_test2) {create(:nud, housing_request: request2, unique_number: "011752019061716361")}
    it "should throw error" do
      expect {request2.is_active = true }.to change {request2.valid?}.from(true).to(false)
    end
  end

  describe "generate als id" do 
    let!(:applicant1) {create(:applicant)}
    let!(:applicant2) {create(:applicant)}
    let!(:hr1) { create(:housing_request, main_applicant: applicant1)}
    let!(:hr2) { create(:housing_request, main_applicant: applicant2)}

    it 'job generate 2 als_id' do
      expect( enqueued_jobs.select {|job| job[:job] == AlsIdUniqJob}.length).to eq(2)
      drain_all_jobs
      expect(hr1.reload.als_id).not_to eq(hr2.reload.als_id)
    end 

    it 'cannot save with same als_id' do 
      Hermes::HousingRequest.remove_indexes
      Hermes::HousingRequest.create_indexes
      expect( enqueued_jobs.select {|job| job[:job] == Hermes::LogJob}.length).to eq(0)
      AlsIdUniqJob.perform_now(hr1.id.to_s, "test")
      AlsIdUniqJob.perform_now(hr2.id.to_s, "test")
      expect(hr1.reload.als_id).to eq("test")
      expect(hr2.reload.als_id).to eq(nil)
      expect( enqueued_jobs.select {|job| job[:job] == Hermes::LogJob && job[:args][0]['code'] == 'HOUSING_REQUEST_GENERATE_START'}.length).to eq(2)
      expect( enqueued_jobs.select {|job| job[:job] == Hermes::LogJob && job[:args][0]['code'] == 'HOUSING_REQUEST_GENERATE_ALS_ID_MAX_TRY'}.length).to eq(1)
      expect( enqueued_jobs.select {|job| job[:job] == Hermes::LogJob && job[:args][0]['code'] == 'HOUSING_REQUEST_GENERATE_RELAUNCH'}.length).to eq(1)
    end 

    it 'not regenerate als_id ' do 
      drain_all_jobs
      als_id = '0000000001'
      expect(hr1.reload.als_id).to eq(als_id)
      10.times do |i|
        AlsIdUniqJob.perform_now(hr1.to_s)
      end
      expect( enqueued_jobs.select {|job| job[:job] == Hermes::LogJob && job[:args][0]['code'] == 'HOUSING_REQUEST_GENERATE_START'}.length).to eq(10)
      expect( enqueued_jobs.select {|job| job[:job] == Hermes::LogJob && job[:args][0]['code'] == 'HOUSING_REQUEST_GENERATE_ALS_ID_MAX_TRY'}.length).to eq(0)
      expect( enqueued_jobs.select {|job| job[:job] == Hermes::LogJob && job[:args][0]['code'] == 'HOUSING_REQUEST_GENERATE_RELAUNCH'}.length).to eq(0)
      expect(hr1.reload.als_id).to eq(als_id)
    end 

  end 

  describe "check spec via csv" do
    path = [Hermes::Engine.root, %w[spec xml_bouchon], "Nomenclature Cerfa v4 - Sheet1.csv"  ].flatten.join('/').to_s
    only_line= false
    cahier_tests_cerfa_v4 = CSV.read(path)
    cahier_tests_cerfa_v4.each_with_index do |line,i|
      if(line[1] && line[2] && line[3] && line[4] && i>0 && (!only_line || i +1 ==only_line))
        describe "test ligne "+ (i+1).to_s + " : " + line[0] + " ----> check if "+line[1]+" is well extracted in xml with unique number "+line[3] do
          let!(:applicant1) {create(:applicant,
            postal_details: {"district"=> "pantin", "address_insee_code"=> "75056"},
            owned_housing_details: {"district"=> "pantin", "address_insee_code"=> "75056"},
            actual_housing_details: {"district"=> "pantin", "address_insee_code"=> "75056"},

            )}
          let!(:housing_request) { create(:housing_request, main_applicant: applicant1,   
            seeked_localisation_list:[{code: "33318", nom: "valette"}],
            postal_details: {"district"=> "pantin", "address_insee_code"=> "75056"},
            owned_housing_details: {"district"=> "pantin", "address_insee_code"=> "75056"},
            actual_housing_details: {"district"=> "pantin", "address_insee_code"=> "75056"},)}
          let!(:nud) {create(:nud, housing_request: housing_request,  unique_number: line[3])}
          before :each do
            housing_request.update_request_from_sne(user)
          end
          it ('should ave value eq to ' + line[4]) do
            path_in_model = line[1].split(".")
            collected_value=housing_request
            if(housing_request.xml)
              if(path_in_model[0] == "coapplicants")
                seached_applicant = Hermes::Applicant.find(housing_request.coapplicants[path_in_model[1].to_i][:id])
                if(path_in_model[2] == "rel_type")
                  collected_value = housing_request.coapplicants[path_in_model[1].to_i][:rel_type]
                else
                  collected_value = seached_applicant.send(path_in_model[2])
                end
              else
               
                path_in_model.each do |key|
                  if collected_value.respond_to?(key)
                    collected_value = collected_value.send(key)
                  elsif (collected_value[key.to_sym] rescue false)
                    collected_value = collected_value[key.to_sym]
                  elsif (collected_value[key.to_i] rescue false)
                    collected_value = collected_value[key.to_i]
                  else
                    collected_value = collected_value[key.to_s]
                  end
                end
              end
              value_to_compare = line[4]
              case line[4] # a_variable is the variable we want to compare
                when "TRUE"    #compare to 1
                  value_to_compare = true
                when "FALSE"    #compare to 2
                  value_to_compare = false
                when "NIL"    #compare to 2
                  value_to_compare = nil
              end
              if( collected_value.is_a?(Date))
                value_to_compare = Date.parse(line[4])
              end
              if( collected_value.is_a?(Integer))
                value_to_compare = line[4].to_i
              end
              expect(collected_value).to eq(value_to_compare)
            else
              ap "le fichier XML "+ line[3] + " est manquant et ne permet pas de réaliser le test n°"+ i.to_s
            end
          end
        end
      end
    end
  end

  describe "update_from_sne" do
    let!(:applicant1) {create(:applicant)}
    let!(:housing_request) { create(:housing_request, main_applicant: applicant1, user: user, consent_level:1)}
    let!(:nud) {create(:nud, housing_request: housing_request, department: nil,  unique_number: "1110121021518G1104")}

    describe "first test" do 
      before :each do

        housing_request.update_request_from_sne(user)
      end
      it 'should add coapplicants to request' do
        # on check qu'on a le bon nombre d'applicant
        expect(housing_request.coapplicants.length).to eq(8)
        expect(Hermes::Applicant.all.length).to eq(9)
        applicant_id = housing_request.coapplicants[0][:id]
        applicant = Hermes::Applicant.find(applicant_id)
        expect(applicant.nationality).to eq("1")
        #on check qu'ils sont bien updatés
        housing_request.main_applicant.main_employer_name = "minie"
        applicant.nationality = "2"
        applicant.main_employer_name = "picsou"
        applicant.save
        expect(applicant.nationality).to eq("2")

        housing_request.update_request_from_sne(user)
        #on check que les données sont updatées
        expect(applicant.reload.nationality).to eq("1")
        #on check que le main employer name est conservé
        expect(applicant.reload.main_employer_name).to eq("picsou")
        expect(housing_request.main_applicant.reload.main_employer_name).to eq("minie")
        expect(housing_request.consent_level).to eq(1)
        expect(applicant.reload.id.to_s).to eq(applicant_id)
        expect(Hermes::Applicant.all.length).to eq(9)
        applicant.first_name = "momo"
        applicant.last_name = "momo"
        applicant.save

        housing_request.reload.update_request_from_sne(user)
        expect(Hermes::Applicant.where(id: applicant.id).length).to eq(0)
        coloc = Hermes::Applicant.find(housing_request.coapplicants[5][:id])
        housing_request.coapplicants.each do |coapp|
          expect(Hermes::Applicant.find(coapp[:id]).user_id).to eq(user._id)
        end
        # Not working
        # coloc.unemployment_period = true
        # coloc.save
        # housing_request.reload.update_request_from_sne(user)
        # expect(coloc.reload.unemployment_period).to eq(true)
        nud.unique_number = "1110121021545G1104"
        nud.save
        housing_request.reload.update_request_from_sne(user)
        housing_request.coapplicants.each do |coapp|
          applic = Hermes::Applicant.find(coapp[:id])
          expect(applic.user_id).to eq(user._id)
        end
        nud.unique_number = "1110121021518G1104"
        nud.save
        housing_request.reload.update_request_from_sne(user)
        housing_request.coapplicants.each do |coapp|
          applic = Hermes::Applicant.find(coapp[:id])
          expect(applic.user_id).to eq(user._id)
        end

        housing_request.main_applicant.workplace_locality = "pantin"
        housing_request.main_applicant.save
        nud.unique_number = "0331120001997G1175"
        nud.save
        housing_request.reload.update_request_from_sne(user)
          expect(housing_request.reload.main_applicant.workplace_locality).to eq(nil)
       
      end
    end
  end

  describe "After actions" do
    let!(:user) { create(:user) }
    let!(:lessor) { create(:lessor) }
    let!(:entity) { create(:entity, lessor: lessor) }
    let!(:housing_offer) { create(:housing_offer, lessor: lessor) }
    let!(:applicant) { create(:applicant, user: user, entity: entity) }  
    let!(:housing_request) { create(:housing_request, is_active: true, main_applicant: applicant, request_status: 'COMPLETE') }
    let!(:housing_application) { create(:housing_application, housing_offer: housing_offer, housing_request: housing_request, status: 'TRANSMIS BAILLEUR' ) }
    
    it "should create public shares if request is applicated" do
      hrs = Hermes::HousingRequestShare.find_by(housing_request_id: housing_request.id, lessor_id: lessor.id)
      expect(hrs).not_to be_nil
    end

    it "should create and link housing_request_folder" do
      folders = Hermes::HousingRequestFolder.where(housing_request_id: housing_request.id)
      expect(housing_request.reload.housing_request_folder_id).not_to be_nil
      expect(folders.count).to eq(1)
    end
  end 

  describe "Entity validations" do
    let!(:user) { create(:user) }
    let!(:lessor) { create(:lessor) }
    let!(:entity) { create(:entity, lessor: lessor, siret: '12345678901234') }
    let!(:lessor2) { create(:lessor) }
    let!(:entity2) { create(:entity, lessor: lessor2, auto_request_validation: true, siret: '22345678901234') }
    let!(:applicant) { create(:applicant, user: user, entity: entity) }
    let!(:housing_request) { create(:housing_request, is_active: true, main_applicant_id: applicant.id, request_status: 'COMPLETE') }
  
    it "should automaticly validate request" do
      applicant.update(main_employer_siret: entity2.siret)
    
      expect(housing_request.reload.is_validated?).to eq(true)
    end
  
    it "should not automaticly validate request" do
      entity2.update(auto_request_validation: false)
      applicant.update(main_employer_siret: entity2.siret)
    
      expect(housing_request.reload.is_validated?).to eq(false)
    end
  
    it 'should auto set entity_status' do
      hr = FactoryBot.create(:housing_request, is_active: true, main_applicant: applicant)
     # expect(hr.entity_status).to eq(nil)

      hr.update(consent_level: 4)
      # expect(hr.entity_status).to eq('EN COURS DE SAISIE')
    end
  end
  describe "Entity_status workflow" do
    let!(:user) { create(:user) }
    let!(:lessor) { create(:lessor) }
    let!(:entity) { create(:entity, lessor: lessor, siret: '00000000000001') }
    let!(:entity2) { create(:entity, lessor: lessor2, auto_request_validation: false, siret: '00000000000002') }
    let!(:lessor2) { create(:lessor) }
    let!(:applicant) { create(:applicant, user: user, main_employer_siret: entity2.siret, amount_year_minus_two: 200000) }
    let!(:housing_request) { create(:housing_request, is_active: true, main_applicant: applicant, consent_level: 4) }
    let!(:nud) { create(:nud,housing_request: housing_request, unique_number: '1')}

    let(:list) do {'nom': 'Fréjus', 'code': '83061', 'codeDepartement': '81', 'codesPostaux': ['83900', '83370', '']} end
    let(:broken_list) do {'code': '77013', 'codePostal': '77013', 'foo': 'bir'} end
  
    describe 'has initial entity_status' do
      it "should have good initial entity_status" do
        expect(housing_request.reload.entity_status).to eq('EN COURS DE SAISIE')
      end
    end
    describe 'has next entity_status' do
      it "should be incomplete because no department" do
        housing_request.validate_all_required_document
        expect(housing_request.reload.entity_status).to eq('A VALIDER')
      end
    end
    describe 'has next entity_status' do
      it "should be complete" do
        housing_request.validate_all_required_document
        housing_request.seeked_localisation_list = [list]
        housing_request.save
        expect(housing_request.reload.entity_status).to eq('A VALIDER')
      end
    end
    # describe 'has next entity_status' do
    #   it "should be incomplete because no departement" do
    #     entity2.auto_request_validation = true
    #     entity2.save
    #     housing_request.validate_all_required_document
    #     expect(housing_request.reload.entity_status).to eq('EN COURS DE SAISIE')
    #   end
    # end
    describe 'has next entity_status' do
      it "should be in progress" do
        entity2.auto_request_validation = true
        entity2.save
        housing_request.validate_all_required_document
        housing_request.seeked_localisation_list = [list]
        housing_request.save
        expect(housing_request.reload.seeked_departments.count).to eq(1)
        expect(housing_request.reload.entity_status).to eq('EN COURS')
      end
    end
    describe 'has next entity_status' do
      it "should be incomplete" do
        housing_request.seeked_localisation_list = [broken_list]
        housing_request.save
        expect(housing_request.reload.entity_status).to eq('EN COURS DE SAISIE')
      end
    end
  end

  describe 'consent_level 1 and entity points' do 
    let!(:entity) { create(:entity, siret: '00000000000001', spent_on_ongoing_requests: 200)}
    let!(:entity2) { create(:entity, siret: '00000000000002', auto_request_validation: false)}
    let!(:entity_user) { create(:user, group_ids: [entity.manager_group_id])}
    let!(:app) { create(:applicant, birth_date: Date.today - 50.years, main_employer_siret: entity.siret, amount_year_minus_two: 11675,  monthly_incomes: {'Salaire': 1345.45}) }
    let!(:hr) { create(:housing_request, main_applicant: app, is_active: true, seeked_localisation_list: [ {codeDepartement: '34'} ], priority_points_indexes: [10],) }
    let!(:nud) { create(:nud, unique_number: '1', housing_request: hr)}
    let!(:entity_form) {create(:entity_form, housing_request_id: hr.id, level: 2) }
    let!(:threshold1) do
      create(
        :threshold,
        type: 'PLA LM',
        thresholds_map: {
          'Paris et communes limitrophes': {
            'thresholds_list': [
              13_050,
              21_272,
              27_883,
              30_521,
              36_307,
              40_859
            ],
            'another_person': 4552
          },
        }
      )
    end
    let!(:offer) { create(:housing_offer,           
      rent_with_charges: 234,
      rent_amount: 234,
      typology: 'T2',
      department: '34',
      commercialized: false,
      threshold_type: 'PLA LM',
      offer_status: 'LOGEMENT PUBLIE',
      threshold_subtype: 'Paris et communes limitrophes'
    ) }


    before do
      hr.validate_all_required_document
    end

    it 'has a link to entity' do
      drain_all_jobs
      hr.reload
      app.reload
      entity.reload
      expect(hr.consent_level).to eq(2)
      expect(hr.request_status).to eq('COMPLETE')
      expect(hr.entity_status).to eq('EN COURS')
      expect(app.entity_id).to eq(entity.id)
      expect(hr.entity_id).to eq(entity.id.to_s)
      expect(app.readable_by?(entity_user)).to eq true
      expect(hr.readable_by?(entity_user)).to eq true
      expect(Hermes::HousingRequest.where(offer.eligibility_query.json_to_h).pluck(:id).include?(hr.id)).to eq true
      expect(entity.spent_on_ongoing_requests).to eq(210)
      expect(entity.spent_on_completed_requests).to eq(0)
      expect(entity.spent_on_other).to eq(0)
      expect(entity.total_spent).to eq(210)
    end 

    it 'lose link to entity when consent_level == 1' do 
      entity_form.update(level: 1)
      drain_all_jobs
      hr.reload
      app.reload
      entity.reload
      expect(hr.consent_level).to eq(1)
      expect(hr.request_status).to eq('COMPLETE')
      expect(hr.entity_status).to eq(nil)
      expect(app.entity_id).to eq(nil)
      expect(app.main_employer_siret).to eq(entity.siret)
      expect(app.old_entity_id).to eq(entity.id)
      expect(hr.entity_id).to eq(nil)
      expect(app.readable_by?(entity_user)).to eq false
      expect(hr.readable_by?(entity_user)).to eq false
      expect(Hermes::HousingRequest.where(offer.eligibility_query.json_to_h).pluck(:id).include?(hr.id)).to eq true
      expect(entity.spent_on_ongoing_requests).to eq(200)
      expect(entity.spent_on_completed_requests).to eq(0)
      expect(entity.spent_on_other).to eq(0)
      expect(entity.total_spent).to eq(200)
    end 

    it 'switch back from consent_level == 1' do 
      entity_form.update(level: 1)
      entity_form.update(level: 3)
      drain_all_jobs
      hr.reload
      app.reload
      entity.reload
      expect(hr.consent_level).to eq(3)
      expect(app.entity_id).to eq(entity.id)
      expect(hr.entity_id).to eq(entity.id.to_s)
      expect(hr.request_status).to eq('COMPLETE')
      expect(hr.entity_status).to eq('EN COURS')
      expect(app.old_entity_id).to eq(entity.id)
      expect(app.readable_by?(entity_user)).to eq true
      expect(hr.readable_by?(entity_user)).to eq true
      expect(Hermes::HousingRequest.where(offer.eligibility_query.json_to_h).pluck(:id).include?(hr.id)).to eq true
      expect(entity.spent_on_ongoing_requests).to eq(210)
      expect(entity.spent_on_completed_requests).to eq(0)
      expect(entity.spent_on_other).to eq(0)
      expect(entity.total_spent).to eq(210)
    end 

    it 'when priority_points_indexes change' do 
      hr.update(priority_points_indexes: [25])
      drain_all_jobs
      hr.reload
      app.reload
      entity.reload
      expect(entity.spent_on_ongoing_requests).to eq(225)
      expect(entity.spent_on_completed_requests).to eq(0)
      expect(entity.spent_on_other).to eq(0)
      expect(entity.total_spent).to eq(225)
    end 

    it 'when entity_id_changed change' do 
      app.update(main_employer_siret: entity2.siret)
      drain_all_jobs
      hr.reload
      app.reload
      entity.reload
      entity2.reload
      expect(entity.spent_on_ongoing_requests).to eq(200)
      expect(entity.spent_on_completed_requests).to eq(0)
      expect(entity.spent_on_other).to eq(0)
      expect(entity.total_spent).to eq(200)
      expect(entity2.spent_on_ongoing_requests).to eq(0)
      expect(entity2.spent_on_completed_requests).to eq(0)
      expect(entity2.spent_on_other).to eq(0)
      expect(entity2.total_spent).to eq(0)
      hr.update(entity_validated: true)
      drain_all_jobs
      hr.reload
      app.reload
      entity.reload
      entity2.reload
      expect(entity.spent_on_ongoing_requests).to eq(200)
      expect(entity.spent_on_completed_requests).to eq(0)
      expect(entity.spent_on_other).to eq(0)
      expect(entity.total_spent).to eq(200)
      expect(entity2.spent_on_ongoing_requests).to eq(10)
      expect(entity2.spent_on_completed_requests).to eq(0)
      expect(entity2.spent_on_other).to eq(0)
      expect(entity2.total_spent).to eq(10)
    end 

    Hermes::HousingRequest.remove_entity_stock_status.each do |type|
      it "entity_status #{type}" do 
        type == 'ANNULE' ? hr.update(request_status: 'ANNULEE') : hr.update(entity_status: type) 
        drain_all_jobs
        hr.reload
        app.reload
        entity.reload
        expect(hr.entity_status).to eq(type)
        expect(entity.spent_on_ongoing_requests).to eq(200)
        expect(entity.spent_on_completed_requests).to eq(0)
        expect(entity.spent_on_other).to eq(0)
        expect(entity.total_spent).to eq(200)
        type == 'ANNULE' ? hr.update(request_status: 'COMPLETE') : hr.update(entity_status: 'EN COURS') 
        drain_all_jobs
        hr.reload
        app.reload
        entity.reload
        expect(entity.spent_on_ongoing_requests).to eq(210)
        expect(entity.spent_on_completed_requests).to eq(0)
        expect(entity.spent_on_other).to eq(0)
        expect(entity.total_spent).to eq(210)
      end 
    end 
  end 

  describe 'Coapplicants Validations' do
    let(:main_applicant) { create(:applicant) }
    let(:housing_request) { create(:housing_request, main_applicant: main_applicant, is_active: true) }
    let(:coapplicant) { create(:applicant) }

    it 'can add coapplicants with blank rel_type' do
      housing_request.update(coapplicants: [{ id: coapplicant.id.to_s, rel_type: '' }])
      expect(housing_request.reload.coapplicants.first[:rel_type]).to be_blank
    end

    it 'can not add coapplicants with blank rel_type if form_status is complete' do
      housing_request.update(form_status: 'complete', coapplicants: [{ id: coapplicant.id.to_s, rel_type: '' }])
      expect(housing_request.errors.any?).to be(true)
      field, error = housing_request.errors.first
      expect(field).to eq(:coapplicants)
      expect(error).to include("rel_type can't be blank for coappllicants")
    end

    it 'can add coapplicants with rel_type if form_status is complete' do
      housing_request.update(form_status: 'complete', coapplicants: [{ id: coapplicant.id.to_s, rel_type: 'C' }])
      expect(housing_request.errors.any?).to be(false)
    end
  end

  describe 'typology_range & min_surface' do
    let(:applicant) { create(:applicant, amount_year_minus_two: 50000) }
    let(:coapplicant) { create(:applicant) }
    let(:hr) { create(:housing_request, is_active: true, main_applicant: applicant ) }

    describe 'right for' do 
      pre_mapping = {
        '1 personne seule' => { typology: 1..2, min_surface: 9, less_limited_dependent_nb: 0, coapplicant_nb: 0 },
        '2 personnes' => { typology: 1..3, min_surface: 16, less_limited_dependent_nb: 0, coapplicant_nb: 1 },
        '2 personnes + 1 enfant à charge' => { typology: 2..4, min_surface: 25, less_limited_dependent_nb: 1, coapplicant_nb: 1 },
        '1 personne seule + 1 enfant à charge' => { typology: 2..3, min_surface: 16, less_limited_dependent_nb: 1, coapplicant_nb: 0 },
        '2 personnes + 2 enfants à charge' => { typology: 3..5, min_surface: 34, less_limited_dependent_nb: 2, coapplicant_nb: 1 },
        '1 personne seule + 2 enfants à charge' => { typology: 2..4, min_surface: 25, less_limited_dependent_nb: 2, coapplicant_nb: 0 },
        '2 personnes + 3 enfants à charge' => { typology: 4..6, min_surface: 43, less_limited_dependent_nb: 3, coapplicant_nb: 1 },
        '1 personne seule + 3 enfants à charge' => { typology: 3..5, min_surface: 34, less_limited_dependent_nb: 3, coapplicant_nb: 0 },
        '2 personnes + 4 enfants à charge' => { typology: 4..7, min_surface: 52, less_limited_dependent_nb: 4, coapplicant_nb: 1 },
        '1 personne seule + 4 enfants à charge' => { typology: 4..6, min_surface: 43, less_limited_dependent_nb: 4, coapplicant_nb: 0 },
        '2 personnes + 5 enfants à charge' => { typology: 5..8, min_surface: 61, less_limited_dependent_nb: 5, coapplicant_nb: 1 },
        '1 personne seule + 5 enfants à charge' => { typology: 5..7, min_surface: 52, less_limited_dependent_nb: 5, coapplicant_nb: 0 },
        '2 personnes + 6 enfants à charge' => { typology: 5..9, min_surface: 70, less_limited_dependent_nb: 6, coapplicant_nb: 1 },
        '1 personne seule + 6 enfants à charge' => { typology: 5..8, min_surface: 61, less_limited_dependent_nb: 6, coapplicant_nb: 0 },
        '2 personnes + 7 enfants à charge' => { typology: 5..10, min_surface: 79, less_limited_dependent_nb: 7, coapplicant_nb: 1 },
        '1 personne seule + 7 enfants à charge' => { typology: 5..9, min_surface: 70, less_limited_dependent_nb: 7, coapplicant_nb: 0 },
        '2 personnes + 8 enfants à charge' => { typology: 5..11, min_surface: 88, less_limited_dependent_nb: 8, coapplicant_nb: 1},
        '1 personne seule + 8 enfants à charge' => { typology: 5..9, min_surface: 70, less_limited_dependent_nb: 7, coapplicant_nb: 0 },
        '2 personnes + 9 enfants à charge' => { typology: 5..11, min_surface: 88, less_limited_dependent_nb: 8, coapplicant_nb: 1},
        '1 personne seule + 9 enfants à charge' => { typology: 5..9, min_surface: 70, less_limited_dependent_nb: 7, coapplicant_nb: 0 },
        '2 personnes + 10 enfants à charge' => { typology: 5..11, min_surface: 88, less_limited_dependent_nb: 8, coapplicant_nb: 1},
      }

      mappings = pre_mapping.each do |k, a|
        pre_mapping[k][:typology] = (a[:typology]).to_a.map do |i|
          "T#{i.to_s}" 
        end
        pre_mapping[k][:typology_filter] = 
          (pre_mapping[k][:typology] & ['T6', 'T7', 'T8']).empty? ? pre_mapping[k][:typology] : pre_mapping[k][:typology] + ['T6P']
      end

      mappings.each do |k, mapping|
        context k do 
          it 'get the good values' do
            hr.dependents = Array.new(mapping[:less_limited_dependent_nb], { test: 'test'}) 
            hr.applicant_ids = mapping[:coapplicant_nb] == 1 ? [coapplicant.id] : [] 
            expect(hr.calculate_typology_range).to eq mapping[:typology]
            expect(hr.calculate_min_surface).to eq mapping[:min_surface]
            hr.typology_range = hr.calculate_typology_range
            expect(hr.typology_filter[:typology][:$in]).to eq mapping[:typology_filter]
          end
        end
      end
      
      it 'unborn_children active' do
        birth_date = (DateTime.now + 150.days).to_s
        hr.update(unborn_children: [
          {"anticipated_birth"=> birth_date, "nb_child_birth"=>"1"},
          {"anticipated_birth"=> birth_date, "nb_child_birth"=>"1"},
        ])
        expect(hr.calculate_unborn_children_nb).to eq 2
        expect(hr.typology_range).to eq mappings['1 personne seule + 2 enfants à charge'][:typology]
        expect(hr.min_surface).to eq mappings['1 personne seule + 2 enfants à charge'][:min_surface]
        expect(hr.main_applicant.eligibility_query.json_to_h[:typology]["$in"]).to eq mappings['1 personne seule + 2 enfants à charge'][:typology]
      end

      it 'unborn_children inactive' do
        birth_date = (DateTime.now + 200.days).to_s
        hr.update(unborn_children: [
          {"anticipated_birth"=> birth_date, "nb_child_birth"=>"1"},
          {"anticipated_birth"=> birth_date, "nb_child_birth"=>"1"},
        ])
        
        expect(hr.calculate_unborn_children_nb).to eq 0
        expect(hr.typology_range).to eq mappings['1 personne seule'][:typology]
        expect(hr.min_surface).to eq mappings['1 personne seule'][:min_surface]
        expect(enqueued_jobs.select {|j| j[:job] == UnbornChildrenJob}.length).to eq 2
        
        Timecop.freeze(DateTime.now + 50.days)
        drain_all_jobs
        expect(hr.calculate_unborn_children_nb).to eq 2
        expect(hr.reload.typology_range).to eq mappings['1 personne seule + 2 enfants à charge'][:typology]
        expect(hr.reload.min_surface).to eq mappings['1 personne seule + 2 enfants à charge'][:min_surface]
        Timecop.return
      end

    end

    describe "inherit birth date from applicant" do
      let!(:applicant) do
        create(
          :applicant,
          birth_date: '2000-01-01'
        )
      end
      let(:housing_request) do
        create(
          :housing_request,
          is_active: true,
          main_applicant: applicant
        )
      end
  
      it 'at creation' do
        expect(housing_request.requester_birth_date).to eq(applicant.birth_date)
      end

      it 'changed applicant birth date' do
        applicant.birth_date = "1900-01-01"
        applicant.save
        applicant.reload
        expect(housing_request.requester_birth_date).to eq(applicant.birth_date)
      end
    end

  end

  describe 'calculate category_type for' do 

    context 'one person' do 
      let(:applicant) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant)}

      it 'as good category' do 
        expect(hr.reload.category_type).to eq(1)
      end 
    end 

    context 'one persone with unborn_children' do 
      let(:applicant) { create(:applicant, birth_date: DateTime.now - 20.years )}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, unborn_children: [{anticipated_birth: (Date.today + 150.days).to_s, nb_child_birth: 1}] )}
      
      it 'as good category' do 
        expect(hr.reload.young_couple?).to eq(false)
        expect(hr.reload.category_type).to eq(1)
      end 
    end 

    context 'two people (not young couple)' do 
      let(:applicant) { create(:applicant, birth_date: DateTime.now - 30.years )}
      let(:coapplicant) { create(:applicant, birth_date: DateTime.now - 30.years)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, applicant_ids: [coapplicant.id.to_s])}
      
      it 'as good category' do 
        expect(hr.reload.category_type).to eq(2)
      end 
    end 
    
    context 'one person with handicap' do 
      let(:applicant) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, disability_details: [{lol: 'test'}])}
      
      it 'as good category' do 
        expect(hr.reload.handicap?).to eq(true)
        expect(hr.reload.category_type).to eq(2)
      end 
    end 

    context 'single_parent with visit' do 
      let(:applicant) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, dependents: [{rel_type: 'D'}])}
      
      it 'as good category' do 
        expect(hr.reload.single_parent).to eq(false)
        expect(hr.reload.category_type).to eq(2)
      end 
    end 

    context '3 people' do 
      let(:applicant) { create(:applicant)}
      let(:coapplicant) { create(:applicant)}
      let(:coapplicant2) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, applicant_ids: [coapplicant.id.to_s, coapplicant2.id.to_s])}
      
      it 'as good category' do 
        expect(hr.reload.category_type).to eq(3)
      end 
    end 

    context '1 person with on dependent' do 
      let(:applicant) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, dependents: [{rel_type: 'E'}])}
      
      it 'as good category' do 
        expect(hr.reload.single_parent).to eq(true)
        expect(hr.reload.category_type).to eq(3)
      end 
    end 

    context '1 person with on dependent parent' do 
      let(:applicant) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, dependents: [{coparentality: 'P'}])}
      
      it 'as good category' do 
        expect(hr.reload.single_parent).to eq(false)
        expect(hr.reload.category_type).to eq(2)
      end 
    end 

    context 'young couple' do 
      let(:applicant) { create(:applicant, birth_date: DateTime.now - 20.years )}
      let(:coapplicant) { create(:applicant, birth_date: DateTime.now - 20.years)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, coapplicants: [{ id: coapplicant.id.to_s, rel_type: 'C'}])}
      
      it 'as good category' do 
        expect(hr.reload.young_couple?).to eq(true)
        expect(hr.reload.category_type).to eq(3)
      end 
    end 

    context '2 people with handicap' do 
      let(:applicant) { create(:applicant)}
      let(:coapplicant) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, applicant_ids: [ coapplicant.id.to_s ], disability_details: [{lol: 'test'}])}
      
      it 'as good category' do 
        expect(hr.reload.handicap?).to eq(true)
        expect(hr.reload.category_type).to eq(3)
      end 
    end 

    context '4 people' do 
      let(:applicant) { create(:applicant)}
      let(:coapplicant) { create(:applicant)}
      let(:coapplicant2) { create(:applicant)}
      let(:coapplicant3) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, applicant_ids: [coapplicant.id.to_s, coapplicant2.id.to_s, coapplicant3.id.to_s])}
      
      it 'as good category' do 
        expect(hr.reload.category_type).to eq(4)
      end 
    end 

    context '1 person with 2 dependents' do 
      let(:applicant) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, dependents: [{rel_type: 'E'}, {rel_type: 'enfant à charge'}])}
      
      it 'as good category' do 
        expect(hr.reload.single_parent).to eq(true)
        expect(hr.reload.category_type).to eq(4)
      end 
    end 

    context '1 person with 2 dependents + one visit' do 
      let(:applicant) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, dependents: [{rel_type: 'E'}, {rel_type: 'enfant à charge'}, {rel_type: 'D'}])}
      
      it 'as good category' do 
        expect(hr.reload.single_parent).to eq(true)
        expect(hr.reload.category_type).to eq(5)
      end 
    end 

    context '2 person with 2 dependents' do 
      let(:applicant) { create(:applicant)}
      let(:coapplicant) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, applicant_ids: [ coapplicant.id.to_s], main_applicant: applicant, dependents: [{rel_type: 'G'}, {rel_type: 'enfant à charge'}])}
      
      it 'as good category' do 
        expect(hr.reload.single_parent).to eq(false)
        expect(hr.reload.category_type).to eq(4)
      end 
    end 

    context '2 person with 2 dependents + one visit' do 
      let(:applicant) { create(:applicant)}
      let(:coapplicant) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, applicant_ids: [ coapplicant.id.to_s], main_applicant: applicant, dependents: [{rel_type: 'D'}, {rel_type: 'enfant à charge'}])}
      
      it 'as good category' do 
        expect(hr.reload.single_parent).to eq(false)
        expect(hr.reload.category_type).to eq(4)
      end 
    end 

    context '3 people with handicap' do 
      let(:applicant) { create(:applicant)}
      let(:coapplicant) { create(:applicant)}
      let(:coapplicant2) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, applicant_ids: [ coapplicant.id.to_s, coapplicant2.id.to_s ], disability_details: [{lol: 'test'}])}
      
      it 'as good category' do 
        expect(hr.reload.handicap?).to eq(true)
        expect(hr.reload.category_type).to eq(4)
      end 
    end 

    context '1 person with 3 dependents' do 
      let(:applicant) { create(:applicant)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, dependents: [{rel_type: 'E'}, {rel_type: 'enfant à charge'}, {rel_type: 'garde alternée'}])}
      
      it 'as good category' do 
        expect(hr.reload.single_parent).to eq(true)
        expect(hr.reload.category_type).to eq(5)
      end 
    end 

    ##########################################

    context 'young couple with 1 children' do 
      let(:applicant) { create(:applicant, birth_date: DateTime.now - 20.years )}
      let(:coapplicant) { create(:applicant, birth_date: DateTime.now - 20.years)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, coapplicants: [{ id: coapplicant.id.to_s, rel_type: 'C'}], dependents: [{rel_type: 'E'}] )}
      
      it 'as good category' do 
        expect(hr.reload.young_couple?).to eq(true)
        expect(hr.reload.category_type).to eq(3)
      end 
    end 

    context 'young couple with 1 parent' do 
      let(:applicant) { create(:applicant, birth_date: DateTime.now - 20.years )}
      let(:coapplicant) { create(:applicant, birth_date: DateTime.now - 20.years)}
      let!(:hr) { create(:housing_request, is_active: true, main_applicant: applicant, coapplicants: [{ id: coapplicant.id.to_s, rel_type: 'C'}], dependents: [{rel_type: 'P'}] )}
      
      it 'as good category' do 
        expect(hr.reload.young_couple?).to eq(true)
        expect(hr.reload.category_type).to eq(3)
      end 
    end 

    context 'young couple with handicap' do 
      let(:applicant) { create(:applicant, birth_date: DateTime.now - 20.years )}
      let(:coapplicant) { create(:applicant, birth_date: DateTime.now - 20.years)}
      let!(:hr) { create(:housing_request,is_active: true, main_applicant: applicant, coapplicants: [{ id: coapplicant.id.to_s, rel_type: 'C'}], disability_details: [{lol: 'test'}] )}
      
      it 'as good category' do 
        expect(hr.reload.young_couple?).to eq(true)
        expect(hr.reload.handicap?).to eq(true)
        expect(hr.reload.category_type).to eq(3)
      end 
    end 

    context 'young couple with changing date' do 
      let(:applicant) { create(:applicant, birth_date: DateTime.now - 20.years )}
      let(:coapplicant) { create(:applicant, birth_date: DateTime.now - 40.years)}
      let!(:hr) { create(:housing_request, is_active: true,  main_applicant: applicant, coapplicants: [{ id: coapplicant.id.to_s, rel_type: 'C'}] )}
      
      it 'as good category' do 
        expect(hr.reload.young_couple?).to eq(false)
        expect(hr.reload.category_type).to eq(2)
        coapplicant.update(birth_date: DateTime.now - 20.years)
        expect(hr.reload.young_couple?).to eq(true)
        expect(hr.reload.category_type).to eq(3)
      end 
    end 
  end 

  describe 'inheriht reference_tax_revenues from coaplicants' do
    let!(:applicant) {create(:applicant)}
    let!(:coapplicant) {create(:applicant)}
    let!(:coapplicant2) {create(:applicant)}
    let!(:hr) {create(:housing_request, main_applicant: applicant, applicant_ids: [coapplicant.id.to_s, coapplicant2.id.to_s], is_active: true, form_status: 'invalid', seeked_departments: ['34'])}
    let!(:nud) { create(:nud,housing_request: hr, unique_number: '1')}

    it 'without amount_year_minus_two' do
      expect(hr.reload.reference_tax_revenues).to eq(nil)
    end

    it 'without amount_year_minus_two 0' do
      applicant.update(amount_year_minus_two: 0)
      expect(hr.reload.reference_tax_revenues).to eq(0)
    end

    it 'only applicant amount_year_minus_two' do
      applicant.update(amount_year_minus_two: 1000)
      expect(hr.reload.reference_tax_revenues).to eq(1000)
    end
    
    it 'all applicants amount_year_minus_two' do
      applicant.update(amount_year_minus_two: 1000)
      coapplicant.update(amount_year_minus_two: 2000)
      expect(hr.reload.reference_tax_revenues).to eq(3000)
      coapplicant2.update(amount_year_minus_two: 500)
      expect(hr.reload.reference_tax_revenues).to eq(3500)
    end

    it 'coapplicant affected after hr creation' do
      coapp = Hermes::Applicant.create(amount_year_minus_two: 800)
      hr.update(applicant_ids: [coapp.id])
      expect(hr.reload.reference_tax_revenues).to eq(800)
    end

    it 'request_status changed' do
      hr.validate_all_required_document
      hr.update(form_status: 'invalid')
      expect(hr.reload.reference_tax_revenues).to eq(nil)
      expect(hr.reload.request_status).to eq("INCOMPLETE")
      
      applicant.update(amount_year_minus_two: 1000)
      hr.update(form_status: 'complete')
      expect(hr.reload.reference_tax_revenues).to eq(1000)
      expect(hr.reload.request_status).to eq("COMPLETE")

      applicant.update(amount_year_minus_two: nil)
      hr.update(form_status: 'invalid')
      expect(hr.reload.reference_tax_revenues).to eq(nil)
      expect(hr.reload.request_status).to eq("INCOMPLETE")

      applicant.update(amount_year_minus_two: 1000)
      hr.update(form_status: 'complete')
      expect(hr.reload.reference_tax_revenues).to eq(1000)
      expect(hr.reload.request_status).to eq("COMPLETE")
    end
  end


  describe 'housing_channel_keys' do 
    let!(:dalo){ create(:housing_channel, key: 'dalo')}
    let!(:lol){ create(:housing_channel, key: 'lol')}
    let!(:applicant){ create(:applicant)}
    let!(:request1) { create(:housing_request, main_applicant: applicant)}
    let!(:request2) { create(:housing_request, main_applicant: applicant, housing_channel_keys: ['dalo'])}

    it 'create with and without housing_channel_keys' do 
      expect(request1.housing_channel_keys).to eq(['dc'])
      expect(request2.housing_channel_keys).to eq(['dalo', 'dc'])
    end 

    it 'when delete housing_channel_keys' do 
      request2.update(housing_channel_keys: [])
      expect(request2.reload.housing_channel_keys).to eq(['dc'])
    end 

    it 'when add housing_channel_keys' do 
      request1.update(housing_channel_keys: ['dalo'])
      expect(request1.reload.housing_channel_keys).to eq(['dalo', 'dc'])
    end 
  end 

  describe 'update_readers_writers' do
    let!(:entity) {create(:entity, siret: '00000000000001')} 
    let!(:entity_user){ create(:user)}
    let!(:user_right) { create(:user_right, type: 'entity', entity: entity, user: entity_user, right: 'administrator')}
    let!(:applicant){ create(:applicant, user: user)}
    let!(:coapp){ create(:applicant, user: user)}
    let!(:request) { create(:housing_request, main_applicant: applicant, is_active: true, user: user, applicant_ids: [coapp.id])}

    it 'setup default' do
      readers = []
      writers = [request.user_id.to_s]
      [applicant.reload, coapp.reload, request.reload].each do |r|
        expect(r.readers).to eq(readers)
        expect(r.writers).to eq(writers)
        expect(request.readable_by?(entity_user)).to eq(false)
      end
    end 

    it 'when entity change' do
      readers = []
      writers = [request.user_id.to_s]
      readers_with_entity = [ "g_" + entity.group_id]
      writers_with_entity = writers + [entity.supervisor_group_id, entity.manager_group_id].map {|id| "g_" + id.to_s}

      applicant.update(main_employer_siret: entity.siret)
      [applicant.reload, coapp.reload, request.reload].each do |r|
        expect(r.readers).to eq(readers_with_entity)
        expect(r.writers).to eq(writers_with_entity)
        expect(request.readable_by?(entity_user)).to eq(true)
      end

      applicant.update(main_employer_siret: nil)
      [applicant.reload, coapp.reload, request.reload].each do |r|
        expect(r.readers).to eq(readers)
        expect(r.writers).to eq(writers)
        expect(request.readable_by?(entity_user)).to eq(false)
      end

    end 

  end 

  describe 'update_request_from_sne' do 
    let!(:applicant) {create(:applicant)}
    let!(:coapplicant) {create(:applicant)}
    let!(:coapps) {  [{:rel_type => 'P', :id => coapplicant.id.to_s}]}
    let!(:housing_request) { create(:housing_request, is_active: true, main_applicant: applicant, coapplicants: coapps)}
    let!(:nud) {create(:nud, unique_number: '1110121021545G1104', housing_request: housing_request)}

    it 'update coapplicant' do 
      expect(housing_request.reload.coapplicants).to eq(coapps)
      housing_request.update_request_from_sne(nil)
      expect(housing_request.reload.coapplicants).to_not eq([])
      expect(housing_request.reload.coapplicants.first[:id]).not_to eq(coapplicant.id.to_s)
    end 

  end 

  describe 'update_request_from_sne update dalo and housing_channel_keys' do
    let!(:dalo){ create(:housing_channel, key: 'dalo')}
    let!(:applicant) {create(:applicant)}
    let!(:hr_with_dalo) {create(:housing_request, main_applicant: applicant)}
    let!(:nud1) { create(:nud,housing_request: hr_with_dalo, unique_number: '1110121047572GDPUB')}
    let!(:hr_without_dalo) {create(:housing_request, main_applicant: applicant)}
    let!(:nud2) { create(:nud,housing_request: hr_without_dalo, unique_number: '1110121021545G1104')}

    it 'works with dalo when empty' do 
      hr_with_dalo.update_request_from_sne(user)
      hr_with_dalo.reload
      expect(hr_with_dalo.dalo).to eq(true)
      expect(hr_with_dalo.housing_channel_keys).to eq([ 'dc', 'dalo'])
    end 

    it 'works with dalo when already set' do 
      hr_with_dalo.update(housing_channel_keys: ['dalo', 'dc'], dalo: true)
      hr_with_dalo.update_request_from_sne(user)
      hr_with_dalo.reload
      expect(hr_with_dalo.dalo).to eq(true)
      expect(hr_with_dalo.housing_channel_keys).to eq(['dalo', 'dc'])
    end 

    it 'works without dalo when empty' do 
      hr_without_dalo.update_request_from_sne(user)
      hr_without_dalo.reload
      expect(hr_without_dalo.dalo).to eq(false)
      expect(hr_without_dalo.housing_channel_keys).to eq(['dc'])
    end 

    it 'works with dalo when already set' do 
      hr_without_dalo.update(housing_channel_keys: ['dalo', 'dc'], dalo: true)
      hr_without_dalo.update_request_from_sne(user)
      hr_without_dalo.reload
      expect(hr_without_dalo.dalo).to eq(false)
      expect(hr_without_dalo.housing_channel_keys).to eq(['dc'])
    end 

    it 'dalo updated when housing_channel_keys change' do 
      expect(hr_with_dalo.reload.dalo).to eq(false)

      hr_with_dalo.update!(housing_channel_keys: ['dalo', 'dc'])
      expect(hr_with_dalo.reload.dalo).to eq(true)

      hr_with_dalo.update!(housing_channel_keys: ['dc'])
      expect(hr_with_dalo.reload.dalo).to eq(false)
    end 

  end 

  describe 'score' do 
    let!(:entity) {create(:entity, siret: '00000000000001', auto_request_validation: false)}
    let!(:applicant) {create(:applicant, birth_date: Date.today - 50.years)}
    let!(:hr) {create(:housing_request, main_applicant: applicant, is_active: true)}
    let!(:nud) {create(:nud, housing_request: hr, unique_number: '1')}
    let!(:offer) {create(:housing_offer)}
    let!(:ha) { create(:housing_application, status: 'EN ATTENTE', housing_request: hr, housing_offer: offer)}

    before do 
      hr.validate_all_required_document
    end 

    it 'work with minimium info' do 
      hr.reload
      expect(hr.profile_score).to eq 0
      expect(hr.entity_points).to eq 0
      expect(hr.motivation_points).to eq 0
      expect(hr.seniority_points).to eq 0
      expect(hr.under_30_year_points).to eq 0
      expect(ha.profile_score).to eq hr.profile_score
    end
    
    it 'update ha score when score change' do 
      expect(ha.reload.profile_score).to eq 0
      hr.update(sne_created_at: DateTime.now - 30.months)
      expect(hr.reload.profile_score).to eq 30
      expect(ha.reload.profile_score).to eq 30
    end 
    
    it 'when priority_points changed' do 
      applicant.update(main_employer_siret: entity.siret)
      hr.update(priority_points_indexes: [100])
      hr.reload
      expect(hr.profile_score).to eq 100
      expect(hr.entity_points).to eq 100
    end 

    it 'when entity_validated changed' do 
      applicant.update(main_employer_siret: entity.siret)
      hr.update(entity_validated: true)
      hr.reload
      expect(hr.profile_score).to eq 240
      expect(hr.entity_points).to eq 240
    end 

    it 'when auto_request_validation changed' do 
      applicant.update(main_employer_siret: entity.siret)
      entity.update(auto_request_validation: true)
      hr.reload
      expect(hr.profile_score).to eq 240
      expect(hr.entity_points).to eq 240
    end 

    it 'when motivations changed' do 
      hr.update(motivations: ['19'])
      hr.reload
      expect(hr.profile_score).to eq 70
      expect(hr.motivation_points).to eq 70
    end 

    it 'when motivations changed' do 
      hr.update(motivations: ['19'])
      hr.reload
      expect(hr.profile_score).to eq 70
      expect(hr.motivation_points).to eq 70
    end 

    it 'when sne_created_at changed' do 
      hr.update(sne_created_at: DateTime.now - 30.months)
      hr.reload
      expect(hr.profile_score).to eq 30
      expect(hr.seniority_points).to eq 30
    end 

    it 'when applicant birth_date changed' do 
      birth_date = Date.today - 20.years
      applicant.update(birth_date: birth_date)
      hr.reload
      expect(hr.requester_birth_date).to eq birth_date
      expect(hr.profile_score).to eq 70
      expect(hr.under_30_year_points).to eq 70
    end 

    it 'when not in progress' do 
      hr.update(motivations: ['19'])
      hr.update(nud_ids: [], unique_numbers: [])
      expect(hr.reload.profile_score).to eq 0
      expect(ha.reload.profile_score).to eq 0
    end 
  end 

  describe 'calculate_next_seniority_updated_date' do 
    let!(:applicant) {create(:applicant)}
    let!(:hr) {create(:housing_request, main_applicant: applicant, is_active: true)}

    [
      {
        title: 'for same day',
        current_date: DateTime.new(2020, 1, 2),
        sne_created_at_date: DateTime.new(2019, 7, 2),
        expected_date: DateTime.new(2020, 2, 3),
        expected_seniority_point: 6,
      },      
      {
        title: 'for same month date',
        current_date: DateTime.new(2020, 1, 2),
        sne_created_at_date: DateTime.new(2019, 7, 15),
        expected_date: DateTime.new(2020, 1, 16),
        expected_seniority_point: 5,
      },
      {
        title: 'for next month',
        current_date: DateTime.new(2020, 1, 22),
        sne_created_at_date: DateTime.new(2019, 7, 15),
        expected_date: DateTime.new(2020, 2, 16),
        expected_seniority_point: 6,
      },
      {
        title: 'for not existing date',
        current_date: DateTime.new(2020, 2, 2),
        sne_created_at_date: DateTime.new(2019, 7, 30),
        expected_date: DateTime.new(2020, 3, 1),
        expected_seniority_point: 6,
      },
    ].each do |d|
      it d[:title] do 
        Timecop.freeze(d[:current_date])
        hr.set(sne_created_at: d[:sne_created_at_date])
        expect(hr.calculate_next_seniority_updated_date).to eq(d[:expected_date].utc)
        expect(hr.calculate_seniority_points).to eq(d[:expected_seniority_point])
        Timecop.return
        Timecop.freeze(d[:expected_date])
        expect(hr.calculate_seniority_points).to eq(d[:expected_seniority_point] + 1)
        Timecop.return
      end 
    end 
  end 

end
